""" LiveDNS commands module. """
import json

from gandi.cli.core.base import GandiModule


class Dns(GandiModule):

    """ Module to handle CLI commands.

    $ gandi dns create
    $ gandi dns delete
    $ gandi dns domain.list
    $ gandi dns list
    $ gandi dns keys create
    $ gandi dns keys delete
    $ gandi dns keys info
    $ gandi dns keys list
    $ gandi dns keys recover

    """

    api_url = 'https://dns.api.gandi.net/api/v5'

    @classmethod
    def get_sort_url(cls, url, sort_by=None):
        if sort_by:
            if not sort_by.startswith('rrset'):
                sort_key = 'rrset_%s' % sort_by
            else:
                sort_key = sort_by
            url = '%s?sort_by=%s' % (url, sort_key)
        return url

    @classmethod
    def list(cls):
        """List domains."""
        return cls.json_get('%s/domains' % cls.api_url)

    @classmethod
    def type_list(cls):
        """List supported records type."""
        return cls.json_get('%s/dns/rrtypes' % cls.api_url)

    @classmethod
    def get_fqdn_info(cls, fqdn):
        """Retrieve information about a domain"""
        return cls.json_get('%s/domains/%s' % (cls.api_url, fqdn))

    @classmethod
    def records(cls, fqdn, sort_by=None, text=False):
        """Display records information about a domain."""
        meta = cls.get_fqdn_info(fqdn)
        url = meta['domain_records_href']
        kwargs = {}
        if text:
            kwargs = {'headers': {'Accept': 'text/plain'}}
        return cls.json_get(cls.get_sort_url(url, sort_by), **kwargs)

    @classmethod
    def add_record(cls, fqdn, name, type, value, ttl):
        """Create record for a domain."""
        data = {
            "rrset_name": name,
            "rrset_type": type,
            "rrset_values": value,
        }
        if ttl:
            data['rrset_ttl'] = int(ttl)
        meta = cls.get_fqdn_info(fqdn)
        url = meta['domain_records_href']
        return cls.json_post(url, data=json.dumps(data))

    @classmethod
    def update_record(cls, fqdn, name, type, value, ttl, content):
        """Update all records for a domain."""
        data = {
            "rrset_name": name,
            "rrset_type": type,
            "rrset_values": value,
        }
        if ttl:
            data['rrset_ttl'] = int(ttl)
        meta = cls.get_fqdn_info(fqdn)
        if content:
            url = meta['domain_records_href']
            kwargs = {'headers': {'Content-Type': 'text/plain'},
                      'data': content}
            return cls.json_put(url, **kwargs)

        url = '%s/domains/%s/records/%s/%s' % (cls.api_url, fqdn, name, type)
        return cls.json_put(url, data=json.dumps(data))

    @classmethod
    def del_record(cls, fqdn, name, type):
        """Delete record for a domain."""
        meta = cls.get_fqdn_info(fqdn)
        url = meta['domain_records_href']
        delete_url = url
        if name:
            delete_url = '%s/%s' % (delete_url, name)
        if type:
            delete_url = '%s/%s' % (delete_url, type)
        return cls.json_delete(delete_url)

    @classmethod
    def keys(cls, fqdn, sort_by=None):
        """Display keys information about a domain."""
        meta = cls.get_fqdn_info(fqdn)
        url = meta['domain_keys_href']
        return cls.json_get(cls.get_sort_url(url, sort_by))

    @classmethod
    def keys_info(cls, fqdn, key):
        """Retrieve key information."""
        return cls.json_get('%s/domains/%s/keys/%s' %
                            (cls.api_url, fqdn, key))

    @classmethod
    def keys_create(cls, fqdn, flag):
        """Create new key entry for a domain."""
        data = {
            "flags": flag,
        }
        meta = cls.get_fqdn_info(fqdn)
        url = meta['domain_keys_href']
        ret, headers = cls.json_post(url, data=json.dumps(data),
                                     return_header=True)
        return cls.json_get(headers['location'])

    @classmethod
    def keys_delete(cls, fqdn, key):
        """Delete a key for a domain."""
        return cls.json_delete('%s/domains/%s/keys/%s' %
                               (cls.api_url, fqdn, key))

    @classmethod
    def keys_recover(cls, fqdn, key):
        """Recover deleted key for a domain."""
        data = {
            "deleted": False,
        }
        return cls.json_put('%s/domains/%s/keys/%s' % (cls.api_url, fqdn, key),
                            data=json.dumps(data),)



# coding: utf-8
"""Contains methods to generate a random password."""

import random
import string

# remove backslash from generated password to avoid
# general escape issues while transmitting it
PUNCTUATION = string.punctuation.replace(chr(0x5c), '')


def mkpassword(length=16, chars=None, punctuation=None):
    """Generates a random ascii string - useful to generate authinfos

    :param length: string wanted length
    :type length: ``int``

    :param chars: character population,
                  defaults to alphabet (lower & upper) + numbers
    :type chars: ``str``, ``list``, ``set`` (sequence)

    :param punctuation: number of punctuation signs to include in string
    :type punctuation: ``int``

    :rtype: ``str``
    """
    if chars is None:
        chars = string.ascii_letters + string.digits

    # Generate string from population
    data = [random.choice(chars) for _ in range(length)]

    # If punctuation:
    # - remove n chars from string
    # - add random punctuation
    # - shuffle chars :)
    if punctuation:
        data = data[:-punctuation]

        for _ in range(punctuation):
            data.append(random.choice(PUNCTUATION))
        random.shuffle(data)

    return ''.join(data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from ..compat import mock
from .base import CommandTestCase
from gandi.cli.commands import record


class RecordTestCase(CommandTestCase):

    def test_list(self):

        result = self.invoke_with_exceptions(record.list, ['iheartcli.com'])

        self.assertEqual(result.output, """\
name        : *
type        : A
value       : 73.246.104.110
ttl         : 10800
----------
name        : @
type        : A
value       : 73.246.104.110
ttl         : 10800
----------
name        : much
type        : A
value       : 192.243.24.132
ttl         : 10800
----------
name        : blog
type        : CNAME
value       : blogs.vip.gandi.net.
ttl         : 10800
----------
name        : cloud
type        : CNAME
value       : gpaas6.dc0.gandi.net.
ttl         : 10800
----------
name        : imap
type        : CNAME
value       : access.mail.gandi.net.
ttl         : 10800
----------
name        : pop
type        : CNAME
value       : access.mail.gandi.net.
ttl         : 10800
----------
name        : smtp
type        : CNAME
value       : relay.mail.gandi.net.
ttl         : 10800
----------
name        : webmail
type        : CNAME
value       : agent.mail.gandi.net.
ttl         : 10800
----------
name        : @
type        : MX
value       : 50 fb.mail.gandi.net.
ttl         : 10800
----------
name        : @
type        : MX
value       : 10 spool.mail.gandi.net.
ttl         : 10800
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_format_text(self):
        args = ['iheartcli.com', '--format', 'text']
        result = self.invoke_with_exceptions(record.list, args)

        self.assertEqual(result.output, """\
* 10800 IN A 73.246.104.110
@ 10800 IN A 73.246.104.110
much 10800 IN A 192.243.24.132
blog 10800 IN CNAME blogs.vip.gandi.net.
cloud 10800 IN CNAME gpaas6.dc0.gandi.net.
imap 10800 IN CNAME access.mail.gandi.net.
pop 10800 IN CNAME access.mail.gandi.net.
smtp 10800 IN CNAME relay.mail.gandi.net.
webmail 10800 IN CNAME agent.mail.gandi.net.
@ 10800 IN MX 50 fb.mail.gandi.net.
@ 10800 IN MX 10 spool.mail.gandi.net.
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_format_json(self):
        args = ['iheartcli.com', '--format', 'json']
        result = self.invoke_with_exceptions(record.list, args)

        self.assertEqual(result.output, """\
[
    {
        "id": 337085079,
        "name": "*",
        "ttl": 10800,
        "type": "A",
        "value": "73.246.104.110"
    },
    {
        "id": 337085078,
        "name": "@",
        "ttl": 10800,
        "type": "A",
        "value": "73.246.104.110"
    },
    {
        "id": 337085081,
        "name": "much",
        "ttl": 10800,
        "type": "A",
        "value": "192.243.24.132"
    },
    {
        "id": 337085072,
        "name": "blog",
        "ttl": 10800,
        "type": "CNAME",
        "value": "blogs.vip.gandi.net."
    },
    {
        "id": 337085082,
        "name": "cloud",
        "ttl": 10800,
        "type": "CNAME",
        "value": "gpaas6.dc0.gandi.net."
    },
    {
        "id": 337085075,
        "name": "imap",
        "ttl": 10800,
        "type": "CNAME",
        "value": "access.mail.gandi.net."
    },
    {
        "id": 337085071,
        "name": "pop",
        "ttl": 10800,
        "type": "CNAME",
        "value": "access.mail.gandi.net."
    },
    {
        "id": 337085074,
        "name": "smtp",
        "ttl": 10800,
        "type": "CNAME",
        "value": "relay.mail.gandi.net."
    },
    {
        "id": 337085073,
        "name": "webmail",
        "ttl": 10800,
        "type": "CNAME",
        "value": "agent.mail.gandi.net."
    },
    {
        "id": 337085077,
        "name": "@",
        "ttl": 10800,
        "type": "MX",
        "value": "50 fb.mail.gandi.net."
    },
    {
        "id": 337085076,
        "name": "@",
        "ttl": 10800,
        "type": "MX",
        "value": "10 spool.mail.gandi.net."
    }
]
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_no_zone(self):

        result = self.invoke_with_exceptions(record.list, ['cli.sexy'])

        self.assertEqual(result.output, """\
No zone records found, domain cli.sexy doesn't seems to be managed at Gandi.
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_output(self):

        args = ['iheartcli.com', '--output']

        with mock.patch('gandi.cli.commands.record.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            result = self.invoke_with_exceptions(record.list, args)

        self.assertEqual(result.output, """\
Your zone file have been writen in iheartcli.com_424242
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_output_reset(self):

        args = ['iheartcli.com', '--output']

        with mock.patch('gandi.cli.commands.record.os.path.isfile',
                        create=True) as mock_isfile:
            mock_isfile.return_value = mock.MagicMock()

            with mock.patch('gandi.cli.commands.record.open',
                            create=True) as mock_open:
                mock_open.return_value = mock.MagicMock()

                result = self.invoke_with_exceptions(record.list, args)

        self.assertEqual(result.output, """\
Your zone file have been writen in iheartcli.com_424242
""")
        self.assertEqual(result.exit_code, 0)

    def test_create_no_zone(self):
        args = ['cli.sexy', '--name', '@', '--type', 'A',
                '--value', '127.0.0.1']
        result = self.invoke_with_exceptions(record.create, args)

        self.assertEqual(result.output, """\
No zone records found, domain cli.sexy doesn't seems to be managed at Gandi.
""")
        self.assertEqual(result.exit_code, 0)

    def test_create(self):
        args = ['iheartcli.com', '--name', '@', '--type', 'A', '--ttl', 3600,
                '--value', '127.0.0.1']
        result = self.invoke_with_exceptions(record.create, args)

        self.assertEqual(result.output, """\
Creating new zone version
Updating zone version
Activation of new zone version
""")
        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['domain.zone.record.add'][0][2]
        self.assertEqual(params['name'], '@')
        self.assertEqual(params['type'], 'A')
        self.assertEqual(params['value'], '127.0.0.1')
        self.assertEqual(params['ttl'], 3600)

    def test_delete_no_zone(self):
        args = ['cli.sexy']
        result = self.invoke_with_exceptions(record.delete, args)

        self.assertEqual(result.output, """\
No zone records found, domain cli.sexy doesn't seems to be managed at Gandi.
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_all_ko(self):
        args = ['iheartcli.com']
        result = self.invoke_with_exceptions(record.delete, args, input='N\n')

        self.assertEqual(result.output, """\
This command without parameters --type, --name or --value will remove all \
records in this zone file. Are you sur to perform this action ? [y/N]: N
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_all(self):
        args = ['iheartcli.com']
        result = self.invoke_with_exceptions(record.delete, args, input='y\n')

        self.assertEqual(result.output, """\
This command without parameters --type, --name or --value will remove all \
records in this zone file. Are you sur to perform this action ? [y/N]: y
Creating new zone record
Deleting zone record
Activation of new zone version
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete(self):
        args = ['iheartcli.com', '--name', '@', '--type', 'A',
                '--value', '127.0.0.1']
        result = self.invoke_with_exceptions(record.delete, args)

        self.assertEqual(result.output, """\
Creating new zone record
Deleting zone record
Activation of new zone version
""")
        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['domain.zone.record.delete'][0][2]
        self.assertEqual(params['name'], '@')
        self.assertEqual(params['type'], 'A')
        self.assertEqual(params['value'], '127.0.0.1')

    def test_update_no_zone(self):
        args = ['cli.sexy']
        result = self.invoke_with_exceptions(record.update, args)

        self.assertEqual(result.output, """\
No zone records found, domain cli.sexy doesn't seems to be managed at Gandi.
""")
        self.assertEqual(result.exit_code, 0)

    def test_update_no_param(self):
        args = ['iheartcli.com']
        result = self.invoke_with_exceptions(record.update, args)

        self.assertEqual(result.output, """\
You must indicate a zone file or a record. \
Use `gandi record update --help` for more information
""")
        self.assertEqual(result.exit_code, 0)

    def test_update(self):
        args = ['iheartcli.com',
                '--record', '* 10800 A 73.246.104.110',
                '--new-record', '@ 3600 A 127.0.0.1']
        result = self.invoke_with_exceptions(record.update, args)

        self.assertEqual(result.output, """\
Creating new zone file
Updating zone records
Activation of new zone version
""")
        self.assertEqual(result.exit_code, 0)

    def test_update_name(self):
        args = ['iheartcli.com',
                '--record', '*',
                '--new-record', '@ 3600 A 127.0.0.1']
        result = self.invoke_with_exceptions(record.update, args)

        self.assertEqual(result.output, """\
Creating new zone file
Updating zone records
Activation of new zone version
""")
        self.assertEqual(result.exit_code, 0)

    def test_update_file(self):
        args = ['iheartcli.com', '--file', 'sandbox/example.txt']

        content = """\
* 10800 IN A 73.246.104.110
@ 10800 IN A 73.246.104.110
much 10800 IN A 192.243.24.132
blog 10800 IN CNAME blogs.vip.gandi.net.
cloud 10800 IN CNAME gpaas6.dc0.gandi.net.
imap 10800 IN CNAME access.mail.gandi.net.
pop 10800 IN CNAME access.mail.gandi.net.
smtp 10800 IN CNAME relay.mail.gandi.net.
webmail 10800 IN CNAME agent.mail.gandi.net.
@ 10800 IN MX 50 fb.mail.gandi.net.
@ 10800 IN MX 10 spool.mail.gandi.net.
"""

        result = self.isolated_invoke_with_exceptions(record.update, args,
                                                      temp_content=content)
        self.assertEqual(result.output, """\
Creating new zone file
Updating zone records
Activation of new zone version
""")
        self.assertEqual(result.exit_code, 0)



""" Configuration namespace commands. """

import sys
import os
import subprocess

import click
from click.exceptions import Abort

from gandi.cli.core.cli import cli
from gandi.cli.core.params import pass_gandi


@cli.group(name='config')
@pass_gandi
def config(gandi):
    """Commands related to Gandi CLI configuration."""


@config.command()
@click.option('-g', help='Get from global configuration only '
                         '(default=env -> local -> global).',
              is_flag=True, default=False)
@click.argument('key')
@pass_gandi
def get(gandi, g, key):
    """Display value of a given config key."""
    val = gandi.get(key=key, global_=g)
    if not val:
        gandi.echo("No value found.")
        sys.exit(1)
    gandi.echo(val)


@config.command()
@click.option('-g', help='Edit global configuration (default=local).',
              is_flag=True, default=False)
@click.argument('key')
@click.argument('value')
@pass_gandi
def set(gandi, g, key, value):
    """Update or create config key/value."""
    gandi.configure(global_=g, key=key, val=value)


@config.command()
@click.option('-g', help='Edit global configuration (default=local).',
              is_flag=True, default=False)
@pass_gandi
def edit(gandi, g):
    """Edit config file with prefered text editor"""
    config_file = gandi.home_config if g else gandi.local_config
    path = os.path.expanduser(config_file)
    editor = gandi.get('editor')
    if not editor:
        try:
            editor = click.prompt("Please enter the path of your prefered "
                                  "editor. eg: '/usr/bin/vi' or 'vi'")
        except Abort:
            gandi.echo("""
Warning: editor is not configured.
You can use both 'gandi config set [-g] editor <value>'
or the $EDITOR environment variable to configure it.""")
            sys.exit(1)

    subprocess.call([editor, path])


@config.command()
@click.option('-g', help='Delete on global configuration (default=local).',
              is_flag=True, default=False)
@click.argument('key')
@pass_gandi
def delete(gandi, g, key):
    """Delete a key/value pair from configuration"""
    gandi.delete(global_=g, key=key)


@config.command()
@click.option('-g', help='Display global configuration (default=local).',
              is_flag=True, default=False)
@pass_gandi
def list(gandi, g):
    """Display config file content"""
    gandi.pretty_echo(gandi.list(global_=g))




def list(options):
    return [{'handle': 'AA1-GANDI'},
            {'handle': 'AA2-GANDI'},
            {'handle': 'AA3-GANDI'},
            {'handle': 'AA4-GANDI'},
            {'handle': 'AA5-GANDI'},
            {'handle': 'TEST1-GANDI'},
            {'handle': 'PXP561-GANDI', 'id': 2920674,
             'prepaid': {'amount': '1337.42',
                         'currency': 'EUR'}}
            ]


def info(id='PXP561-GANDI'):

    contact = dict([(contact['handle'], contact) for contact in list({})])
    return contact[id]


def create(params):
    return {'handle': 'PP0000-GANDI'}


def create_dry_run(params):

    errors = []
    if params['phone'] == '555-123-456':
        errors.append({'attr': None,
                       'error': '!EC_STRMATCH',
                       'field': 'phone',
                       'field_type': 'String',
                       'reason': "phone: string '555-123-456' does not "
                                 "match '^\\+\\d{1,3}\\.\\d+$'"})

    if params['email'] == 'green.goblin@spiderman.org':
        # add an unknown error
        errors.append({'attr': ['Sun, Mercury, Venus, Earth, Mars, Jupiter,'
                                'Saturn, Uranus, Neptune'],
                       'error': '!EC_ENUMIN',
                       'field': 'planet',
                       'field_type': 'Enum',
                       'reason': 'planet: Pluto not in list Sun, Mercury, '
                                 'Venus, Earth, Mars, Jupiter, '
                                 'Saturn, Uranus, Neptune'})

    return errors


def balance(id='PXP561-GANDI'):

    contact = dict([(contact['handle'], contact) for contact in list({})])
    return contact[id]



""" Status commands module. """

try:
    import urllib.parse as uparse
except ImportError:
    import urllib as uparse

from gandi.cli.core.base import GandiModule


class Status(GandiModule):

    """ Module to handle CLI commands.

    $ gandi status

    """

    base_url = 'https://status.gandi.net'
    api_url = 'https://status.gandi.net/api'

    @classmethod
    def descriptions(cls):
        """ Retrieve status descriptions from status.gandi.net. """
        schema = cls.json_get('%s/status/schema' % cls.api_url, empty_key=True,
                              send_key=False)
        descs = {}
        for val in schema['fields']['status']['value']:
            descs.update(val)
        return descs

    @classmethod
    def services(cls):
        """Retrieve services statuses from status.gandi.net."""
        return cls.json_get('%s/services' % cls.api_url, empty_key=True,
                            send_key=False)

    @classmethod
    def status(cls):
        """Retrieve global status from status.gandi.net."""
        return cls.json_get('%s/status' % cls.api_url, empty_key=True,
                            send_key=False)

    @classmethod
    def events(cls, filters):
        """Retrieve events details from status.gandi.net."""
        current = filters.pop('current', False)
        current_params = []
        if current:
            current_params = [('current', 'true')]

        filter_url = uparse.urlencode(sorted(list(filters.items())) + current_params)  # noqa
        events = cls.json_get('%s/events?%s' % (cls.api_url, filter_url),
                              empty_key=True, send_key=False)
        return events

    @classmethod
    def event_timeline(cls, event):
        """Retrieve event timeline url for status.gandi.net."""
        return '%s/timeline/events/%s' % (cls.base_url, event['id'])



# -*- coding: utf-8 -*-

import re

from .base import CommandTestCase
from gandi.cli.commands import vlan
from gandi.cli.core.base import GandiContextHelper


class VlanTestCase(CommandTestCase):

    def test_list(self):

        result = self.invoke_with_exceptions(vlan.list, [])

        self.assertEqual(result.output, """\
name      : vlantest
state     : created
datacenter: FR-SD2
----------
name      : pouet
state     : created
datacenter: FR-SD2
----------
name      : intranet
state     : created
datacenter: FR-SD3
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_filters(self):
        args = ['--id', '--subnet', '--gateway']
        result = self.invoke_with_exceptions(vlan.list, args)

        self.assertEqual(result.output, """\
name      : vlantest
state     : created
id        : 123
subnet    : 10.7.13.0/24
gateway   : 10.7.13.254
datacenter: FR-SD2
----------
name      : pouet
state     : created
id        : 717
subnet    : 192.168.232.0/24
gateway   : 192.168.232.254
datacenter: FR-SD2
----------
name      : intranet
state     : created
id        : 999
subnet    : 10.7.242.0/24
gateway   : 10.7.242.254
datacenter: FR-SD3
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_filter_datacenter(self):
        args = ['--id', '--subnet', '--gateway', '--datacenter', 'FR-SD3']
        result = self.invoke_with_exceptions(vlan.list, args)

        self.assertEqual(result.output, """\
name      : intranet
state     : created
id        : 999
subnet    : 10.7.242.0/24
gateway   : 10.7.242.254
datacenter: FR-SD3
""")
        self.assertEqual(result.exit_code, 0)

    def test_info(self):
        args = ['vlantest']
        result = self.invoke_with_exceptions(vlan.info, args)

        self.assertEqual(result.output, """\
name       : vlantest
state      : created
subnet     : 10.7.13.0/24
gateway    : 10.7.13.254
datacenter : FR-SD2
""")
        self.assertEqual(result.exit_code, 0)

    def test_info_ip(self):
        args = ['pouet', '--ip']
        result = self.invoke_with_exceptions(vlan.info, args)

        self.assertEqual(result.output, """\
name       : pouet
state      : created
subnet     : 192.168.232.0/24
gateway    : 192.168.232.254 don't exists
datacenter : FR-SD2
----------
bandwidth  : 102400.0
vm         : server02
ip         : 192.168.232.252
----------
bandwidth  : 204800.0
vm         : server02
ip         : 192.168.232.253
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete(self):
        args = ['intranet']
        result = self.invoke_with_exceptions(vlan.delete, args, input='y\n',
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Are you sure to delete vlan 'intranet'? [y/N]: y
Deleting your vlan.
\rProgress: [###] 100.00%  00:00:00""")

        self.assertEqual(result.exit_code, 0)

    def test_delete_force(self):
        args = ['intranet', '--force']
        result = self.invoke_with_exceptions(vlan.delete, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Deleting your vlan.
\rProgress: [###] 100.00%  00:00:00""")

        self.assertEqual(result.exit_code, 0)

    def test_delete_background(self):
        args = ['intranet', '--force', '--bg']
        result = self.invoke_with_exceptions(vlan.delete, args,
                                             obj=GandiContextHelper())

        self.assertEqual(result.output, """\
id        : 200
step      : WAIT
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_unknown(self):
        args = ['vlanunknown']
        result = self.invoke_with_exceptions(vlan.delete, args)

        self.assertEqual(result.output, """\
Sorry vlan vlanunknown does not exist
Please use one of the following: ['vlantest', 'pouet', 'intranet', \
'123', '717', '999']
""")

        self.assertEqual(result.exit_code, 0)

    def test_delete_refuse(self):
        args = ['intranet']
        result = self.invoke_with_exceptions(vlan.delete, args, input='\n',
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Are you sure to delete vlan 'intranet'? [y/N]:""")

        self.assertEqual(result.exit_code, 0)

    def test_create(self):
        args = ['--name', 'testvlan', '--datacenter', 'FR-SD3',
                '--subnet', '10.7.70.0/24', '--gateway', '10.7.70.254']
        result = self.invoke_with_exceptions(vlan.create, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Creating your vlan.
\rProgress: [###] 100.00%  00:00:00  \
\nYour vlan testvlan has been created.""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.vlan.create'][0][0]
        self.assertEqual(params['datacenter_id'], 4)
        self.assertEqual(params['subnet'], '10.7.70.0/24')
        self.assertEqual(params['name'], 'testvlan')
        self.assertEqual(params['gateway'], '10.7.70.254')

    def test_create_datacenter_limited(self):
        args = ['--name', 'testvlan', '--datacenter', 'FR-SD2',
                '--subnet', '10.7.70.0/24', '--gateway', '10.7.70.254']
        result = self.invoke_with_exceptions(vlan.create, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
/!\ Datacenter FR-SD2 will be closed on 25/12/2017, please consider using \
another datacenter.
Creating your vlan.
\rProgress: [###] 100.00%  00:00:00  \
\nYour vlan testvlan has been created.""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.vlan.create'][0][0]
        self.assertEqual(params['datacenter_id'], 1)
        self.assertEqual(params['subnet'], '10.7.70.0/24')
        self.assertEqual(params['name'], 'testvlan')
        self.assertEqual(params['gateway'], '10.7.70.254')

    def test_create_datacenter_closed(self):
        args = ['--name', 'testvlan', '--datacenter', 'US-BA1',
                '--subnet', '10.7.70.0/24', '--gateway', '10.7.70.254']
        result = self.invoke_with_exceptions(vlan.create, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Error: /!\ Datacenter US-BA1 is closed, please choose another datacenter.""")

        self.assertEqual(result.exit_code, 1)

    def test_create_background(self):
        args = ['--name', 'testvlanbg', '--bg']
        result = self.invoke_with_exceptions(vlan.create, args,
                                             obj=GandiContextHelper())

        self.assertEqual(result.output.strip(), """\
{'id': 200, 'step': 'WAIT'}""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.vlan.create'][0][0]
        self.assertEqual(params['datacenter_id'], 5)
        self.assertEqual(params['name'], 'testvlanbg')

    def test_update(self):
        args = ['pouet', '--name', 'chocolat',
                '--gateway', '10.7.70.254',
                '--bandwidth', '204800']

        result = self.invoke_with_exceptions(vlan.update, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Updating your vlan.""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.vlan.update'][0][1]
        self.assertEqual(params['name'], 'chocolat')
        self.assertEqual(params['gateway'], '10.7.70.254')

    def test_update_gateway_vm_unknown(self):
        args = ['pouet', '--name', 'chocolat',
                '--gateway', 'server01',
                '--bandwidth', '204800']

        result = self.invoke_with_exceptions(vlan.update, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Can't find 'server01' in 'pouet' vlan""")

        self.assertEqual(result.exit_code, 0)

    def test_update_gateway_vm(self):
        args = ['pouet', '--name', 'chocolat',
                '--gateway', 'server01',
                '--create',
                '--bandwidth', '204800']

        result = self.invoke_with_exceptions(vlan.update, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Will create a new ip in this vlan for vm server01
Creating your iface.
\rProgress: [###] 100.00%  00:00:00  \
\nYour iface has been created with the following IP addresses:
ip4:\t95.142.160.181
ip6:\t2001:4b98:dc0:47:216:3eff:feb2:3862
Attaching your iface.
\rProgress: [###] 100.00%  00:00:00  \
\nUpdating your vlan.""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.vlan.update'][0][1]
        self.assertEqual(params['name'], 'chocolat')
        self.assertEqual(params['gateway'], '95.142.160.181')

    def test_update_gateway_multiple_ips(self):
        args = ['pouet', '--name', 'chocolat',
                '--gateway', 'server02']

        result = self.invoke_with_exceptions(vlan.update, args,
                                             obj=GandiContextHelper())

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
This vm has two ips in the vlan, don't know which one to choose \
(213.167.231.3, 192.168.232.252)""")

        self.assertEqual(result.exit_code, 0)



import json
from functools import partial
from ..compat import mock
from .base import CommandTestCase
from gandi.cli.commands import root

# disable SSL requests warning for tests
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()


RESPONSES = {
    'https://status.gandi.net/api/status/schema': {
        'status': 200,
        'headers': 'application/json',
        'body': """{"fields": {"status": {"value": [
                   {"SUNNY": "All services are up and running"},
                   {"CLOUDY": "A scheduled maintenance ongoing"},
                   {"FOGGY": "Incident which are not impacting our services."},
                   {"STORMY": "An incident ongoing"}]}}}"""
    },
}


def _mock_requests(status, method, url, *args, **kwargs):
    content = None
    if status == 'SUNNY':
        if url == 'https://status.gandi.net/api/services':
            content = """[{"description": "IAAS",
                           "name": "IAAS",
                           "status": "SUNNY"},
                          {"description": "PAAS",
                           "name": "PAAS",
                           "status": "SUNNY"},
                          {"description": "Site",
                          "name": "Site",
                          "status": "SUNNY"},
                          {"description": "API",
                          "name": "API",
                          "status": "SUNNY"},
                          {"description": "SSL",
                          "name": "SSL",
                          "status": "SUNNY"},
                          {"description": "Domain",
                          "name": "Domain",
                          "status": "SUNNY"},
                          {"description": "Email",
                          "name": "Email",
                          "status": "SUNNY"}]"""
        if url == 'https://status.gandi.net/api/events?category=Incident&current=true':  # noqa
            content = """[]"""

    if status == 'STORMY':
        if url == 'https://status.gandi.net/api/services':
            content = """[{"description": "IAAS",
                           "name": "IAAS",
                           "status": "SUNNY"},
                          {"description": "PAAS",
                           "name": "PAAS",
                           "status": "STORMY"},
                          {"description": "Site",
                          "name": "Site",
                          "status": "SUNNY"},
                          {"description": "API",
                          "name": "API",
                          "status": "SUNNY"},
                          {"description": "SSL",
                          "name": "SSL",
                          "status": "SUNNY"},
                          {"description": "Domain",
                          "name": "Domain",
                          "status": "SUNNY"},
                          {"description": "Email",
                          "name": "Email",
                          "status": "SUNNY"}]"""
        if url == 'https://status.gandi.net/api/events?category=Incident&services=PAAS&current=true':  # noqa
            content = """
                [{"category": "Incident",
                "date_end": "2014-10-08T06:20:00+00:00",
                "date_start": "2014-10-07T18:00:00+00:00",
                "duration": 740,
                "estimate_date_end": "2014-10-08T06:20:00+00:00",
                "id": "7",
                "services": [
                    "IAAS",
                    "PAAS"
                ],
                "title": "Incident on a storage unit on Paris datacenter"}]
                """

    if status == 'FOGGY':
        if url == 'https://status.gandi.net/api/services':
            content = """[{"description": "IAAS",
                           "name": "IAAS",
                           "status": "SUNNY"},
                          {"description": "PAAS",
                           "name": "PAAS",
                           "status": "SUNNY"},
                          {"description": "Site",
                          "name": "Site",
                          "status": "SUNNY"},
                          {"description": "API",
                          "name": "API",
                          "status": "SUNNY"},
                          {"description": "SSL",
                          "name": "SSL",
                          "status": "SUNNY"},
                          {"description": "Domain",
                          "name": "Domain",
                          "status": "SUNNY"},
                          {"description": "Email",
                          "name": "Email",
                          "status": "SUNNY"}]"""
        if url == 'https://status.gandi.net/api/events?category=Incident&current=true':  # noqa
            content = """
                [{"category": "Incident",
                "date_end": "2015-04-15T22:06:43+00:00",
                "date_start": "2015-04-15T21:30:00+00:00",
                "duration": 36,
                "estimate_date_end": "2015-04-15T22:30:00+00:00",
                "id": "15",
                "services": [],
                "title": "Reachability issue on our website"
                }]
                """

    if url == 'https://status.gandi.net/api/status':
        content = """{"status": "%s"}""" % status
    elif not content:
        content = RESPONSES[url]['body']

    content = json.loads(content)
    mock_resp = mock.Mock()
    mock_resp.status_code = 200
    mock_resp.content = content
    mock_resp.json = mock.Mock(return_value=content)
    return mock_resp


class StatusTestCase(CommandTestCase):

    @mock.patch('gandi.cli.core.client.requests.request')
    def test_status(self, mock_request):
        mock_request.side_effect = partial(_mock_requests, 'SUNNY')

        result = self.invoke_with_exceptions(root.status, [])

        wanted = ("""\
IAAS      : All services are up and running
PAAS      : All services are up and running
Site      : All services are up and running
API       : All services are up and running
SSL       : All services are up and running
Domain    : All services are up and running
Email     : All services are up and running
""")

        self.assertEqual(result.output, wanted)

        self.assertEqual(result.exit_code, 0)

    @mock.patch('gandi.cli.core.client.requests.request')
    def test_status_service(self, mock_request):
        mock_request.side_effect = partial(_mock_requests, 'SUNNY')

        result = self.invoke_with_exceptions(root.status, ['ssl'])

        wanted = ("""\
SSL       : All services are up and running
""")

        self.assertEqual(result.output, wanted)

        self.assertEqual(result.exit_code, 0)

    @mock.patch('gandi.cli.core.client.requests.request')
    def test_status_service_incident(self, mock_request):
        mock_request.side_effect = partial(_mock_requests, 'STORMY')

        result = self.invoke_with_exceptions(root.status, ['paas'])

        url = 'https://status.gandi.net/timeline/events/7'
        wanted = ("""\
PAAS      : Incident on a storage unit on Paris datacenter - %s
""") % url

        self.assertEqual(result.output, wanted)

        self.assertEqual(result.exit_code, 0)

    @mock.patch('gandi.cli.core.client.requests.request')
    def test_status_no_service_incident(self, mock_request):
        mock_request.side_effect = partial(_mock_requests, 'FOGGY')

        result = self.invoke_with_exceptions(root.status, [])

        wanted = ("""\
Reachability issue on our website - https://status.gandi.net/timeline/events/15
IAAS      : All services are up and running
PAAS      : All services are up and running
Site      : All services are up and running
API       : All services are up and running
SSL       : All services are up and running
Domain    : All services are up and running
Email     : All services are up and running
""")

        self.assertEqual(result.output, wanted)

        self.assertEqual(result.exit_code, 0)



# -*- coding: utf-8 -*-
""" Contains GandiModule and GandiContextHelper classes.

GandiModule class is used by commands modules.
GandiContextHelper class is used as click context for commands.
"""

from __future__ import print_function
import os
import sys
import time
import os.path
from datetime import datetime
from subprocess import check_call, Popen, PIPE, CalledProcessError

import click
from click.exceptions import UsageError

from .client import XMLRPCClient, APICallFailed, DryRunException, JsonClient
from .conf import GandiConfig


class MissingConfiguration(Exception):

    """ Raise when no configuration was found. """


class GandiModule(GandiConfig):

    """ Base class for modules.

    Manage
    - initializing xmlrpc connection
    - execute remote api calls
    """

    _op_scores = {'BILL': 0, 'WAIT': 1, 'RUN': 2, 'DONE': 3}

    verbose = 0
    _api = None
    # frequency of api calls when polling for operation progress
    _poll_freq = 1

    @classmethod
    def get_api_connector(cls):
        """ Initialize an api connector for future use."""
        if cls._api is None:  # pragma: no cover
            cls.load_config()
            cls.debug('initialize connection to remote server')
            apihost = cls.get('api.host')
            if not apihost:
                raise MissingConfiguration()

            apienv = cls.get('api.env')
            if apienv and apienv in cls.apienvs:
                apihost = cls.apienvs[apienv]

            cls._api = XMLRPCClient(host=apihost, debug=cls.verbose)

        return cls._api

    @classmethod
    def call(cls, method, *args, **kwargs):
        """ Call a remote api method and return the result."""
        api = None
        empty_key = kwargs.pop('empty_key', False)
        try:
            api = cls.get_api_connector()
            apikey = cls.get('api.key')
            if not apikey and not empty_key:
                cls.echo("No apikey found, please use 'gandi setup' "
                         "command")
                sys.exit(1)
        except MissingConfiguration:
            if api and empty_key:
                apikey = ''
            elif not kwargs.get('safe'):
                cls.echo("No configuration found, please use 'gandi setup' "
                         "command")
                sys.exit(1)
            else:
                return []

        # make the call
        cls.debug('calling method: %s' % method)
        for arg in args:
            cls.debug('with params: %r' % arg)
        try:
            resp = api.request(method, apikey, *args,
                               **{'dry_run': kwargs.get('dry_run', False),
                                  'return_dry_run':
                                  kwargs.get('return_dry_run', False)})
            cls.dump('responded: %r' % resp)
            return resp
        except APICallFailed as err:
            if kwargs.get('safe'):
                return []
            if err.code == 530040:
                cls.echo("Error: It appears you haven't purchased any credits "
                         "yet.\n"
                         "Please visit https://www.gandi.net/credit/buy to "
                         "learn more and buy credits.")
                sys.exit(1)
            if err.code == 510150:
                cls.echo("Invalid API key, please use 'gandi setup' command.")
                sys.exit(1)
            if isinstance(err, DryRunException):
                if kwargs.get('return_dry_run', False):
                    return err.dry_run
                else:
                    for msg in err.dry_run:
                        # TODO use trads with %s
                        cls.echo(msg['reason'])
                        cls.echo('\t' + ' '.join(msg['attr']))
                    sys.exit(1)
            error = UsageError(err.errors)
            setattr(error, 'code', err.code)
            raise error

    @classmethod
    def safe_call(cls, method, *args):
        """ Call a remote api method but don't raise if an error occurred."""
        return cls.call(method, *args, safe=True)

    @classmethod
    def json_call(cls, method, url, **kwargs):
        """ Call a remote api using json format """
        # retrieve api key if needed
        empty_key = kwargs.pop('empty_key', False)
        send_key = kwargs.pop('send_key', True)
        return_header = kwargs.pop('return_header', False)
        try:
            apikey = cls.get('apirest.key')
            if not apikey and not empty_key:
                cls.echo("No apikey found for REST API, please use "
                         "'gandi setup' command")
                sys.exit(1)
            if send_key:
                if 'headers' in kwargs:
                    kwargs['headers'].update({'X-Api-Key': apikey})
                else:
                    kwargs['headers'] = {'X-Api-Key': apikey}
        except MissingConfiguration:
            if not empty_key:
                return []
        # make the call
        cls.debug('calling url: %s %s' % (method, url))
        cls.debug('with params: %r' % kwargs)
        try:
            resp, resp_headers = JsonClient.request(method, url, **kwargs)
            cls.dump('responded: %r' % resp)
            if return_header:
                return resp, resp_headers
            return resp
        except APICallFailed as err:
            cls.echo('An error occured during call: %s' % err.errors)
            sys.exit(1)

    @classmethod
    def json_get(cls, url, **kwargs):
        """ Helper for GET json request """
        return cls.json_call('GET', url, **kwargs)

    @classmethod
    def json_post(cls, url, **kwargs):
        """ Helper for POST json request """
        return cls.json_call('POST', url, **kwargs)

    @classmethod
    def json_put(cls, url, **kwargs):
        """ Helper for PUT json request """
        return cls.json_call('PUT', url, **kwargs)

    @classmethod
    def json_delete(cls, url, **kwargs):
        """ Helper for DELETE json request """
        return cls.json_call('DELETE', url, **kwargs)

    @classmethod
    def intty(cls):
        """ Check if we are in a tty. """
        # XXX: temporary hack until we can detect if we are in a pipe or not
        return True

        if hasattr(sys.stdout, 'isatty') and sys.stdout.isatty():
            return True

        return False

    @classmethod
    def echo(cls, message):
        """ Display message. """
        if cls.intty():
            if message is not None:
                print(message)

    @classmethod
    def pretty_echo(cls, message):
        """ Display message using pretty print formatting. """
        if cls.intty():
            if message:
                from pprint import pprint
                pprint(message)

    @classmethod
    def separator_line(cls, sep='-', size=10):
        """ Display a separator line. """
        if cls.intty():
            cls.echo(sep * size)

    @classmethod
    def separator_sub_line(cls, sep='-', size=10):
        """ Display a separator line. """
        if cls.intty():
            cls.echo("\t" + sep * size)

    @classmethod
    def dump(cls, message):
        """ Display dump message if verbose level allows it. """
        if cls.verbose > 2:
            msg = '[DUMP] %s' % message
            cls.echo(msg)

    @classmethod
    def debug(cls, message):
        """ Display debug message if verbose level allows it. """
        if cls.verbose > 1:
            msg = '[DEBUG] %s' % message
            cls.echo(msg)

    @classmethod
    def log(cls, message):
        """ Display info message if verbose level allows it. """
        if cls.verbose > 0:
            msg = '[INFO] %s' % message
            cls.echo(msg)

    @classmethod
    def deprecated(cls, message):
        print('[deprecated]: %s' % message, file=sys.stderr)

    @classmethod
    def error(cls, msg):
        """ Raise click UsageError exception using msg. """
        raise UsageError(msg)

    @classmethod
    def execute(cls, command, shell=True):
        """ Execute a shell command. """
        cls.debug('execute command (shell flag:%r): %r ' % (shell, command))
        try:
            check_call(command, shell=shell)
            return True
        except CalledProcessError:
            return False

    @classmethod
    def exec_output(cls, command, shell=True, encoding='utf-8'):
        """ Return execution output

        :param encoding: charset used to decode the stdout
        :type encoding: str

        :return: the return of the command
        :rtype: unicode string
        """
        proc = Popen(command, shell=shell, stdout=PIPE)
        stdout, _stderr = proc.communicate()
        if proc.returncode == 0:
            return stdout.decode(encoding)

        return ''

    @classmethod
    def update_progress(cls, progress, starttime):
        """ Display an ascii progress bar while processing operation. """
        width, _height = click.get_terminal_size()
        if not width:
            return

        duration = datetime.utcnow() - starttime
        hours, remainder = divmod(duration.seconds, 3600)
        minutes, seconds = divmod(remainder, 60)

        size = int(width * .6)
        status = ""
        if isinstance(progress, int):
            progress = float(progress)
        if not isinstance(progress, float):
            progress = 0
            status = 'error: progress var must be float\n'
            cls.echo(type(progress))
        if progress < 0:
            progress = 0
            status = 'Halt...\n'
        if progress >= 1:
            progress = 1
            # status = 'Done...\n'
        block = int(round(size * progress))
        text = ('\rProgress: [{0}] {1:.2%} {2} {3:0>2}:{4:0>2}:{5:0>2}  '
                ''.format('#' * block + '-' * (size - block), progress,
                          status, hours, minutes, seconds))
        sys.stdout.write(text)
        sys.stdout.flush()

    @classmethod
    def display_progress(cls, operations):
        """ Display progress of Gandi operations.

        polls API every 1 seconds to retrieve status.
        """
        start_crea = datetime.utcnow()

        # count number of operations, 3 steps per operation
        if not isinstance(operations, (list, tuple)):
            operations = [operations]
        count_operations = len(operations) * 3
        updating_done = False
        while not updating_done:
            op_score = 0
            for oper in operations:
                op_ret = cls.call('operation.info', oper['id'])
                op_step = op_ret['step']
                if op_step in cls._op_scores:
                    op_score += cls._op_scores[op_step]
                else:
                    cls.echo('')
                    msg = 'step %s unknown, exiting' % op_step
                    if op_step == 'ERROR':
                        msg = ('An error has occured during operation '
                               'processing: %s' % op_ret['last_error'])
                    elif op_step == 'SUPPORT':
                        msg = ('An error has occured during operation '
                               'processing, you must contact Gandi support.')
                    cls.echo(msg)
                    sys.exit(1)

            cls.update_progress(float(op_score) / count_operations,
                                start_crea)

            if op_score == count_operations:
                updating_done = True

            time.sleep(cls._poll_freq)

        cls.echo('\r')


class GandiContextHelper(GandiModule):

    """ Gandi context helper.

    Import module classes from modules directory upon initialization.
    """

    _modules = {}

    def __init__(self, verbose=-1):
        """ Initialize variables and api connection. """
        GandiModule.verbose = verbose
        GandiModule.load_config()
        # only load modules once
        if not self._modules:
            self.load_modules()

    def __getattribute__(self, item):
        """ Return module from internal imported modules dict. """
        if item in object.__getattribute__(self, '_modules'):
            return object.__getattribute__(self, '_modules')[item]
        return object.__getattribute__(self, item)

    def load_modules(self):
        """ Import CLI commands modules. """
        module_folder = os.path.join(os.path.dirname(__file__),
                                     '..', 'modules')
        module_dirs = {
            'gandi.cli': module_folder
        }

        if 'GANDICLI_PATH' in os.environ:
            for _path in os.environ.get('GANDICLI_PATH').split(':'):
                # remove trailing separator if any
                path = _path.rstrip(os.sep)
                module_dirs[os.path.basename(path)] = os.path.join(path,
                                                                   'modules')

        for module_basename, dir in list(module_dirs.items()):
            for filename in sorted(os.listdir(dir)):
                if filename.endswith('.py') and '__init__' not in filename:
                    submod = filename[:-3]
                    module_name = module_basename + '.modules.' + submod
                    __import__(module_name, fromlist=[module_name])

        # save internal map of loaded module classes
        for subclass in GandiModule.__subclasses__():
            self._modules[subclass.__name__.lower()] = subclass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from .base import CommandTestCase
from gandi.cli.commands import webacc


class WebaccTestCase(CommandTestCase):

    def test_list(self):
        result = self.invoke_with_exceptions(webacc.list, [])

        self.assertEqual(result.output, """\
name          : webacc01
state         : running
ssl           : Disable
Vhosts :
Backends :
\tBackend with ip address 195.142.160.181 no longer exists.
\tYou should remove it.
\tip            : 195.142.160.181
\tport          : 80
\tstate         : running

----
name          : testwebacc
state         : running
ssl           : Disable
Vhosts :
\tvhost         : pouet.iheartcli.com
\tssl           : Disable

Backends :
\tname          : server01
\tip            : 95.142.160.181
\tport          : 80
\tstate         : running

""")
        self.assertEqual(result.exit_code, 0)

    def test_list_output_json(self):
        args = ['--format', 'json']
        result = self.invoke_with_exceptions(webacc.list, args)

        self.assertEqual(result.output, """\
[{"datacenter_id": 3, "date_created": "20160115T162658", "id": 12138, \
"name": "webacc01", "probe": {"enable": true, "host": null, "interval": null, \
"method": null, "response": null, "threshold": null, "timeout": null, \
"url": null, "window": null}, "servers": [{"fallback": false, "id": 14988, \
"ip": "195.142.160.181", "port": 80, "rproxy_id": 132691, \
"state": "running"}], "ssl_enable": false, "state": "running", \
"uuid": 12138, "vhosts": []}, {"datacenter_id": 1, \
"date_created": "20160115T162658", "id": 13263, "name": "testwebacc", \
"probe": {"enable": true, "host": "95.142.160.181", "interval": 10, \
"method": "GET", "response": 200, "threshold": 3, "timeout": 5, "url": "/", \
"window": 5}, "servers": [{"fallback": false, "id": 4988, \
"ip": "95.142.160.181", "port": 80, "rproxy_id": 13269, "state": "running"}], \
"ssl_enable": false, "state": "running", "uuid": 13263, \
"vhosts": [{"cert_id": null, "id": 5171, "name": "pouet.iheartcli.com", \
"rproxy_id": 13263, "state": "running"}]}]
""")
        self.assertEqual(result.exit_code, 0)

    def test_info(self):
        result = self.invoke_with_exceptions(webacc.info, ['testwebacc'])

        self.assertEqual(result.output, """\
name          : testwebacc
state         : running
datacenter    : Equinix Paris
ssl           : Disable
algorithm     : client-ip
Vhosts :
\tvhost         : pouet.iheartcli.com
\tssl           : None

Backends :
\tname          : server01
\tip            : 95.142.160.181
\tport          : 80
\tstate         : running

Probe :
\tstate         : Enabled
\thost          : 95.142.160.181
\tinterval      : 10
\tmethod        : GET
\tresponse      : 200
\tthreshold     : 3
\ttimeout       : 5
\turl           : /
\twindow        : 5
""")
        self.assertEqual(result.exit_code, 0)

    def test_info_output_json(self):
        args = ['testwebacc', '--format', 'json']
        result = self.invoke_with_exceptions(webacc.info, args)

        self.assertEqual(result.output, """\
{"datacenter": {"country": "France", "dc_code": "FR-SD2", "id": 1, \
"iso": "FR", "name": "Equinix Paris"}, "date_created": "20160115T162658", \
"id": 13263, "lb": {"algorithm": "client-ip"}, "name": "testwebacc", \
"probe": {"enable": true, "host": "95.142.160.181", "interval": 10, \
"method": "GET", "response": 200, "threshold": 3, "timeout": 5, "url": "/", \
"window": 5}, "servers": [{"fallback": false, "id": 4988, \
"ip": "95.142.160.181", "port": 80, "rproxy_id": 13269, "state": "running"}], \
"ssl_enable": false, "state": "running", "uuid": 13263, \
"vhosts": [{"cert_id": null, "id": 5171, "name": "pouet.iheartcli.com", \
"rproxy_id": 13263, "state": "running"}]}
""")
        self.assertEqual(result.exit_code, 0)

    def test_info_backend_expired(self):
        result = self.invoke_with_exceptions(webacc.info, ['webacc01'])

        self.assertEqual(result.output, """\
name          : webacc01
state         : running
datacenter    : Equinix Paris
ssl           : Disable
algorithm     : client-ip
Vhosts :
Backends :
\tBackend with ip address 195.142.160.181 no longer exists.
\tYou should remove it.
\tip            : 195.142.160.181
\tport          : 80
\tstate         : running

Probe :
\tstate         : Enabled
\thost          :
\tinterval      :
\tmethod        :
\tresponse      :
\tthreshold     :
\ttimeout       :
\turl           :
\twindow        :
""")
        self.assertEqual(result.exit_code, 0)

    def test_create(self):
        args = ['webacc2', '--datacenter', 'FR-SD3']
        result = self.invoke_with_exceptions(webacc.create, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Creating your webaccelerator webacc2
\rProgress: [###] 100.00%  00:00:00  \
\nYour webaccelerator have been created""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.rproxy.create'][0][0]
        self.assertEqual(params['name'], 'webacc2')
        self.assertEqual(params['datacenter_id'], 4)
        self.assertEqual(params['ssl_enable'], False)
        self.assertEqual(params['lb'], {'algorithm': 'client-ip'})
        self.assertEqual(params['zone_alter'], False)
        self.assertEqual(params['override'], True)

    def test_create_datacenter_limited(self):
        args = ['webacc2', '--datacenter', 'FR-SD2']
        result = self.invoke_with_exceptions(webacc.create, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
/!\ Datacenter FR-SD2 will be closed on 25/12/2017, please consider using \
another datacenter.
Creating your webaccelerator webacc2
\rProgress: [###] 100.00%  00:00:00  \
\nYour webaccelerator have been created""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.rproxy.create'][0][0]
        self.assertEqual(params['name'], 'webacc2')
        self.assertEqual(params['datacenter_id'], 1)
        self.assertEqual(params['ssl_enable'], False)
        self.assertEqual(params['lb'], {'algorithm': 'client-ip'})
        self.assertEqual(params['zone_alter'], False)
        self.assertEqual(params['override'], True)

    def test_create_datacenter_closed(self):
        args = ['webacc2', '--datacenter', 'US-BA1']
        result = self.invoke_with_exceptions(webacc.create, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Error: /!\ Datacenter US-BA1 is closed, please choose another datacenter.""")

        self.assertEqual(result.exit_code, 1)

    def test_create_ssl_ok(self):
        args = ['webacc2', '--datacenter', 'FR-SD3', '--vhost',
                'pouet.lol.cat', '--ssl']
        result = self.invoke_with_exceptions(webacc.create, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please give the private key for certificate id 710 (CN: lol.cat)""")

        self.assertEqual(result.exit_code, 0)

    def test_create_backend_prompt(self):
        args = ['webacc2', '--datacenter', 'FR-SD3',
                '--backend', '195.142.160.181']
        result = self.invoke_with_exceptions(webacc.create, args,
                                             input='8080\n')

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please set a port for backends. \
If you want to set different port for each backend, use `-b ip:port`: 8080
Creating your webaccelerator webacc2
\rProgress: [###] 100.00%  00:00:00  \
\nYour webaccelerator have been created""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.rproxy.create'][0][0]
        self.assertEqual(params['name'], 'webacc2')
        self.assertEqual(params['datacenter_id'], 4)
        self.assertEqual(params['ssl_enable'], False)
        self.assertEqual(params['lb'], {'algorithm': 'client-ip'})
        self.assertEqual(params['zone_alter'], False)
        self.assertEqual(params['override'], True)
        self.assertEqual(params['servers'], ({'ip': u'195.142.160.181',
                                              'port': 8080},))

    def test_create_backend_port_ok(self):
        args = ['webacc2', '--datacenter', 'FR-SD3',
                '--backend', '195.142.160.181', '--port', 9000]
        result = self.invoke_with_exceptions(webacc.create, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Creating your webaccelerator webacc2
\rProgress: [###] 100.00%  00:00:00  \
\nYour webaccelerator have been created""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.rproxy.create'][0][0]
        self.assertEqual(params['name'], 'webacc2')
        self.assertEqual(params['datacenter_id'], 4)
        self.assertEqual(params['ssl_enable'], False)
        self.assertEqual(params['lb'], {'algorithm': 'client-ip'})
        self.assertEqual(params['zone_alter'], False)
        self.assertEqual(params['override'], True)
        self.assertEqual(params['servers'], ({'ip': u'195.142.160.181',
                                              'port': 9000},))

    def test_update(self):
        args = ['testwebacc', '-n', 'testwebacc2',
                '--ssl-enable', '--algorithm', 'round-robin']
        result = self.invoke_with_exceptions(webacc.update, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Updating your webaccelerator
\rProgress: [###] 100.00%  00:00:00  \
\nThe webaccelerator have been udated""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.rproxy.update'][0][1]
        self.assertEqual(params['name'], 'testwebacc2')
        self.assertEqual(params['ssl_enable'], True)
        self.assertEqual(params['lb'], {'algorithm': 'round-robin'})

    def test_delete_webacc(self):
        args = ['-w', 'webacc01']
        result = self.invoke_with_exceptions(webacc.delete, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Deleting your webaccelerator named webacc01
\rProgress: [###] 100.00%  00:00:00  \
\nWebaccelerator have been deleted""")

        self.assertEqual(result.exit_code, 0)

    def test_delete_vhost(self):
        args = ['-v', 'pouet.iheartcli.com']
        result = self.invoke_with_exceptions(webacc.delete, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Deleting your virtual host pouet.iheartcli.com
\rProgress: [###] 100.00%  00:00:00  \
\nYour virtual host have been removed""")

        self.assertEqual(result.exit_code, 0)

    def test_delete_backend_prompt(self):
        args = ['--backend', '195.142.160.181']
        result = self.invoke_with_exceptions(webacc.delete, args,
                                             input='8080\n')

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please set a port for backends. \
If you want to  different port for each backend, use `-b ip:port`: 8080
Removing backend 195.142.160.181:8080 into webaccelerator
\rProgress: [###] 100.00%  00:00:00  \
\nYour backend have been removed""")

        self.assertEqual(result.exit_code, 0)

    def test_delete_backend_port_ok(self):
        args = ['--backend', '195.142.160.181', '--port', 9000]
        result = self.invoke_with_exceptions(webacc.delete, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Removing backend 195.142.160.181:9000 into webaccelerator
\rProgress: [###] 100.00%  00:00:00  \
\nYour backend have been removed""")

        self.assertEqual(result.exit_code, 0)

    def test_enable_probe(self):
        args = ['webacc01', '-p']
        result = self.invoke_with_exceptions(webacc.enable, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Activating probe on webacc01
\rProgress: [###] 100.00%  00:00:00  \
\nThe probe have been activated""")

        self.assertEqual(result.exit_code, 0)

    def test_enable_probe_no_resource(self):
        args = ['-p']
        result = self.invoke_with_exceptions(webacc.enable, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
You need to indicate the Webaccelerator name""")

        self.assertEqual(result.exit_code, 0)

    def test_enable_backend_prompt(self):
        args = ['webacc01', '--backend', '195.142.160.181']
        result = self.invoke_with_exceptions(webacc.enable, args,
                                             input='8080\n')
        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please set a port for backends. \
If you want to  different port for each backend, use `-b ip:port`: 8080
Activating backend 195.142.160.181
\rProgress: [###] 100.00%  00:00:00  \
\nBackend activated""")

        self.assertEqual(result.exit_code, 0)

    def test_enable_backend_port_ok(self):
        args = ['webacc01', '--backend', '195.142.160.181',
                '--port', 9000]
        result = self.invoke_with_exceptions(webacc.enable, args)
        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Activating backend 195.142.160.181
\rProgress: [###] 100.00%  00:00:00  \
\nBackend activated""")

        self.assertEqual(result.exit_code, 0)

    def test_disable_probe(self):
        args = ['webacc01', '-p']
        result = self.invoke_with_exceptions(webacc.disable, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Desactivating probe on webacc01
\rProgress: [###] 100.00%  00:00:00  \
\nThe probe have been desactivated""")

        self.assertEqual(result.exit_code, 0)

    def test_disable_probe_no_resource(self):
        args = ['-p']
        result = self.invoke_with_exceptions(webacc.disable, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
You need to indicate the Webaccelerator name""")

        self.assertEqual(result.exit_code, 0)

    def test_disable_backend_port_ok(self):
        args = ['webacc01', '--backend', '195.142.160.181',
                '--port', 9000]
        result = self.invoke_with_exceptions(webacc.disable, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Desactivating backend on server 195.142.160.181
\rProgress: [###] 100.00%  00:00:00  \
\nBackend desactivated""")

        self.assertEqual(result.exit_code, 0)

    def test_disable_backend_prompt(self):
        args = ['webacc01', '--backend', '195.142.160.181']
        result = self.invoke_with_exceptions(webacc.disable, args,
                                             input='8080\n')

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please set a port for backends. \
If you want to  different port for each backend, use `-b ip:port`: 8080
Desactivating backend on server 195.142.160.181
\rProgress: [###] 100.00%  00:00:00  \
\nBackend desactivated""")

        self.assertEqual(result.exit_code, 0)

    def test_add_vhost(self):
        args = ['webacc01', '-v', 'pouet.iheartcli.com', '--zone-alter',
                '--ssl']
        result = self.invoke_with_exceptions(webacc.add, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
There is no certificate for pouet.iheartcli.com.
Create the certificate with (for exemple) :
$ gandi certificate create --cn pouet.iheartcli.com --type std \
\nOr relaunch the current command with --poll-cert option
Adding your virtual host (pouet.iheartcli.com) into webacc01
\rProgress: [###] 100.00%  00:00:00  \
\nYour virtual host habe been added""")

        self.assertEqual(result.exit_code, 0)
        params = self.api_calls['hosting.rproxy.vhost.create'][0][1]
        self.assertEqual(params['vhost'], 'pouet.iheartcli.com')
        self.assertEqual(params['zone_alter'], True)

    def test_add_vhost_ssl_ok(self):
        args = ['webacc01', '-v', 'pouet.lol.cat', '--zone-alter',
                '--ssl']
        result = self.invoke_with_exceptions(webacc.add, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please give the private key for certificate id 710 (CN: lol.cat)""")

        self.assertEqual(result.exit_code, 0)

    def test_add_backend_prompt(self):
        args = ['webacc01', '-b', '195.142.160.181']
        result = self.invoke_with_exceptions(webacc.add, args,
                                             input='80\n')

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Please set a port for backends. \
If you want to  different port for each backend, use `-b ip:port`: 80
Adding backend 195.142.160.181:80 into webaccelerator
\rProgress: [###] 100.00%  00:00:00  \
\nBackend added""")

        self.assertEqual(result.exit_code, 0)

    def test_add_backend_port_ok(self):
        args = ['webacc01', '-b', '195.142.160.181', '--port', 9000]
        result = self.invoke_with_exceptions(webacc.add, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Adding backend 195.142.160.181:9000 into webaccelerator
\rProgress: [###] 100.00%  00:00:00  \
\nBackend added""")

        self.assertEqual(result.exit_code, 0)

    def test_probe_test(self):
        args = ['webacc01', '--window', 5,
                '--timeout', 5, '--threshold', 3, '--interval', 10,
                '--host', '95.142.160.181', '--url', '/',
                '--http-method', 'GET', '--http-response', 200,
                '--test']
        result = self.invoke_with_exceptions(webacc.probe, args)

        self.assertEqual(result.output.strip(), """\
status        : 200
timeout       : 1.0""")
        self.assertEqual(result.exit_code, 0)

    def test_probe_update(self):
        args = ['webacc01', '--window', 5,
                '--timeout', 5, '--threshold', 3, '--interval', 10,
                '--host', '95.142.160.181', '--url', '/',
                '--http-method', 'GET', '--http-response', 200]
        result = self.invoke_with_exceptions(webacc.probe, args)

        self.assertEqual(re.sub(r'\[#+\]', '[###]',
                                result.output.strip()), """\
Progress: [###] 100.00%  00:00:00""")

        self.assertEqual(result.exit_code, 0)



""" Virtual hosts namespace commands. """

import click

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import output_generic, output_vhost
from gandi.cli.core.params import pass_gandi


@cli.group(name='vhost')
@pass_gandi
def vhost(gandi):
    """Commands related to Simple Hosting virtual hosts."""


@vhost.command()
@click.option('--limit', help='Limit number of results.', default=100,
              show_default=True)
@click.option('--id', help='Display ids.', is_flag=True)
@click.option('--names', help='Display names.', is_flag=True)
@pass_gandi
def list(gandi, limit, id, names):
    """ List vhosts. """
    options = {
        'items_per_page': limit,
    }

    output_keys = ['name', 'state', 'date_creation']
    if id:
        # When we will have more than paas vhost, we will append rproxy_id
        output_keys.append('paas_id')

    paas_names = {}
    if names:
        output_keys.append('paas_name')
        paas_names = gandi.paas.list_names()

    result = gandi.vhost.list(options)
    for num, vhost in enumerate(result):
        paas = paas_names.get(vhost['paas_id'])
        if num:
            gandi.separator_line()
        output_vhost(gandi, vhost, paas, output_keys)

    return result


@vhost.command()
@click.option('--id', help='Display ids.', is_flag=True)
@click.argument('resource', nargs=-1, required=True)
@pass_gandi
def info(gandi, resource, id):
    """ Display information about a vhost.

    Resource must be the vhost fqdn.
    """
    output_keys = ['name', 'state', 'date_creation', 'paas_name', 'ssl']

    if id:
        # When we will have more than paas vhost, we will append rproxy_id
        output_keys.append('paas_id')

    paas_names = gandi.paas.list_names()

    ret = []
    paas = None
    for num, item in enumerate(resource):
        vhost = gandi.vhost.info(item)
        try:
            hostedcert = gandi.hostedcert.infos(vhost['name'])
            vhost['ssl'] = 'activated' if hostedcert else 'disabled'
        except ValueError:
            vhost['ssl'] = 'disabled'
        paas = paas_names.get(vhost['paas_id'])
        if num:
            gandi.separator_line()
        ret.append(output_vhost(gandi, vhost, paas, output_keys))

    return ret


@vhost.command()
@click.option('--paas', required=True,
              help='PaaS instance on which to create it.')
@click.option('--ssl', help='Get ssl on that vhost.', is_flag=True)
@click.option('--private-key', '--pk',
              help='Private key used to generate the ssl Certificate.')
@click.option('--alter-zone', help='Will update the domain zone.',
              is_flag=True)
@click.option('--poll-cert', help='Will wait for the certificate creation.',
              is_flag=True)
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.argument('vhost', required=True)
@pass_gandi
def create(gandi, vhost, paas, ssl, private_key, alter_zone, poll_cert,
           background):
    """ Create a new vhost. """
    if not gandi.hostedcert.activate_ssl(vhost, ssl, private_key, poll_cert):
        return

    paas_info = gandi.paas.info(paas)
    result = gandi.vhost.create(paas_info, vhost, alter_zone, background)

    if background:
        gandi.pretty_echo(result)

    return result


@vhost.command()
@click.option('--ssl', help='Get ssl on that vhost.', is_flag=True)
@click.option('--private-key', '--pk',
              help='Private key used to generate the ssl Certificate.')
@click.option('--poll-cert', help='Will wait for the certificate creation.',
              is_flag=True)
@click.argument('resource', nargs=1, required=True)
@pass_gandi
def update(gandi, resource, ssl, private_key, poll_cert):
    """ Update a vhost.

    Right now you can only activate ssl on the vhost.
    """
    gandi.hostedcert.activate_ssl(resource, ssl, private_key, poll_cert)


@vhost.command()
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.argument('resource', required=True)
@pass_gandi
def delete(gandi, resource, force, background):
    """ Delete a vhost. """
    output_keys = ['name', 'paas_id', 'state', 'date_creation']
    if not force:
        proceed = click.confirm('Are you sure to delete vhost %s?' %
                                resource)

        if not proceed:
            return

    opers = gandi.vhost.delete(resource, background)
    if background:
        for oper in opers:
            output_generic(gandi, oper, output_keys)

    return opers



import os
from click.testing import CliRunner

from gandi.cli.core.base import GandiModule
from ..compat import unittest, mock
from ..fixtures.api import Api
from ..fixtures.mocks import MockObject


class CommandTestCase(unittest.TestCase):

    base_mocks = [
        ('gandi.cli.core.base.GandiModule.save', MockObject.blank_func),
        ('gandi.cli.core.base.GandiModule.execute', MockObject.execute),
        ('gandi.cli.core.base.GandiModule.deprecated', MockObject.deprecated),
    ]
    mocks = []

    def setUp(self):
        self.runner = CliRunner()

        self.mocks = self.mocks + self.base_mocks
        self.mocks = [mock.patch(*mock_args) for mock_args in self.mocks]
        for dummy in self.mocks:
            dummy.start()

        GandiModule._api = Api()
        GandiModule._api._calls = {}
        GandiModule._conffiles = {'global': {'api': {'env': 'test',
                                                     'key': 'apikey0001'},
                                             'apirest': {'key': 'apikey002'}}}
        GandiModule._poll_freq = 0.1

        self.api_calls = GandiModule._api._calls

    def tearDown(self):
        GandiModule._api = None
        GandiModule._conffiles = {}

        for dummy in reversed(self.mocks):
            dummy.stop()

    def invoke_with_exceptions(self, cli, args, catch_exceptions=False,
                               **kwargs):
        return self.runner.invoke(cli, args, catch_exceptions=catch_exceptions,
                                  **kwargs)

    def isolated_invoke_with_exceptions(self, cli, args,
                                        catch_exceptions=False,
                                        temp_dir=None,
                                        temp_name=None,
                                        temp_content=None,
                                        **kwargs):

        temp_dir = temp_dir or 'sandbox'
        temp_name = temp_name or 'example.txt'
        with self.runner.isolated_filesystem():
            os.mkdir(temp_dir)

            with open('%s/%s' % (temp_dir, temp_name), 'w') as f:
                f.write(temp_content)

            return self.runner.invoke(cli, args,
                                      catch_exceptions=catch_exceptions,
                                      **kwargs)



""" Snapshot profile commands module. """

from gandi.cli.core.base import GandiModule
from gandi.cli.core.utils import DuplicateResults


class SnapshotProfile(GandiModule):

    """ Module to handle CLI commands.

    $ gandi snapshotprofile info
    $ gandi snapshotprofile list

    """

    @classmethod
    def from_name(cls, name):
        """ Retrieve a snapshot profile accsociated to a name."""
        snps = cls.list({'name': name})
        if len(snps) == 1:
            return snps[0]['id']
        elif not snps:
            return

        raise DuplicateResults('snapshot profile name %s is ambiguous.' % name)

    @classmethod
    def usable_id(cls, id):
        """ Retrieve id from input which can be name or id."""
        try:
            qry_id = cls.from_name(id)
            if not qry_id:
                qry_id = int(id)
        except DuplicateResults as exc:
            cls.error(exc.errors)
        except Exception:
            qry_id = None

        if not qry_id:
            msg = 'unknown identifier %s' % id
            cls.error(msg)

        return qry_id

    @classmethod
    def list(cls, options=None, target=None):
        """ List all snapshot profiles."""
        options = options or {}

        result = []
        if not target or target == 'paas':
            for profile in cls.safe_call('paas.snapshotprofile.list', options):
                profile['target'] = 'paas'
                result.append((profile['id'], profile))

        if not target or target == 'vm':
            for profile in cls.safe_call('hosting.snapshotprofile.list',
                                         options):
                profile['target'] = 'vm'
                result.append((profile['id'], profile))

        result = sorted(result, key=lambda item: item[0])
        return [profile for id_, profile in result]

    @classmethod
    def info(cls, resource):
        """Display information about a snapshot profile."""
        snps = cls.list({'id': cls.usable_id(resource)})
        if len(snps) == 1:
            return snps[0]
        elif not snps:
            return

        raise DuplicateResults('snapshot profile %s is ambiguous.' % resource)



""" Record commands module. """

from gandi.cli.core.base import GandiModule


class Zone(GandiModule):

    """ Helper class for domain DNS zones. """

    @classmethod
    def new(cls, zone_id):
        """Create a new zone version."""
        return cls.call('domain.zone.version.new', zone_id)

    @classmethod
    def set(cls, zone_id, version_id):
        """Set active version of a zone."""
        return cls.call('domain.zone.version.set', zone_id, version_id)

    @classmethod
    def delete(cls, zone_id, version_id):
        """Delete a version of a zone."""
        return cls.call('domain.zone.version.delete', zone_id, version_id)


class Record(GandiModule):

    """ Module to handle CLI commands.

    $ gandi record list
    $ gandi record create

    """

    @classmethod
    def list(cls, zone_id, options=None):
        """List zone records for a zone."""
        options = options if options else {}
        return cls.call('domain.zone.record.list', zone_id, 0, options)

    @classmethod
    def add(cls, zone_id, version_id, record):
        """Add record to a zone."""
        return cls.call('domain.zone.record.add', zone_id, version_id, record)

    @classmethod
    def create(cls, zone_id, record):
        """Create a new zone version for record."""
        cls.echo('Creating new zone version')
        new_version_id = Zone.new(zone_id)

        cls.echo('Updating zone version')
        cls.add(zone_id, new_version_id, record)

        cls.echo('Activation of new zone version')
        Zone.set(zone_id, new_version_id)

        return new_version_id

    @classmethod
    def delete(cls, zone_id, record):
        """Delete a record for a zone"""
        cls.echo('Creating new zone record')
        new_version_id = Zone.new(zone_id)

        cls.echo('Deleting zone record')
        cls.call('domain.zone.record.delete', zone_id, new_version_id, record)

        cls.echo('Activation of new zone version')
        Zone.set(zone_id, new_version_id)

        return new_version_id

    @classmethod
    def zone_update(cls, zone_id, records):
        """Update records for a zone"""
        cls.echo('Creating new zone file')
        new_version_id = Zone.new(zone_id)

        cls.echo('Updating zone records')
        cls.call('domain.zone.record.set', zone_id, new_version_id, records)

        cls.echo('Activation of new zone version')
        Zone.set(zone_id, new_version_id)

        return new_version_id

    @classmethod
    def update(cls, zone_id, old_record, new_record):
        """Update a record in a zone file"""
        cls.echo('Creating new zone file')
        new_version_id = Zone.new(zone_id)

        new_record = new_record.replace(' IN', '')
        new_record = new_record.split(' ', 4)
        params_newrecord = {'name': new_record[0], 'ttl': int(new_record[1]),
                            'type': new_record[2], 'value': new_record[3]}

        old_record = old_record.replace(' IN', '')
        old_record = old_record.split(' ', 4)
        try:
            params = {'name': old_record[0], 'ttl': int(old_record[1]),
                      'type': old_record[2], 'value': old_record[3]}
        except IndexError:
            # failed to retrieve all values, try only use the name
            params = {'name': old_record[0]}
        record = cls.call('domain.zone.record.list', zone_id, new_version_id,
                          params)

        if record:
            cls.echo('Updating zone records')
            try:
                cls.call('domain.zone.record.update', zone_id,
                         new_version_id, {'id': record[0]['id']},
                         params_newrecord)
                cls.echo('Activation of new zone version')
                Zone.set(zone_id, new_version_id)
                return new_version_id
            except Exception as err:
                cls.echo('An error as occured: %s' % err)
                Zone.delete(zone_id, new_version_id)
        else:
            cls.echo('The record to update does not exist. Check records'
                     ' already created with `gandi record list example.com'
                     ' --output`')
            return False



try:
    # python3
    from xmlrpc.client import DateTime
except ImportError:
    # python2
    from xmlrpclib import DateTime


type_list = list


def package_list(options):
    return [{'category': {'id': 1, 'name': 'standard'},
             'comodo_id': 287,
             'id': 1,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_std_1_0_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 0,
             'wildcard': 0},
            {'category': {'id': 1, 'name': 'standard'},
             'comodo_id': 279,
             'id': 2,
             'max_domains': 3,
             'min_domains': 1,
             'name': 'cert_std_3_0_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 0,
             'wildcard': 0},
            {'category': {'id': 1, 'name': 'standard'},
             'comodo_id': 289,
             'id': 3,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_std_w_0_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 0,
             'wildcard': 1},
            {'category': {'id': 2, 'name': 'pro'},
             'comodo_id': 24,
             'id': 4,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_pro_1_10_0',
             'sgc': 0,
             'trustlogo': 1,
             'warranty': 10000,
             'wildcard': 0},
            {'category': {'id': 2, 'name': 'pro'},
             'comodo_id': 34,
             'id': 5,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_pro_1_100_0',
             'sgc': 0,
             'trustlogo': 1,
             'warranty': 100000,
             'wildcard': 0},
            {'category': {'id': 2, 'name': 'pro'},
             'comodo_id': 317,
             'id': 6,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_pro_1_100_SGC',
             'sgc': 1,
             'trustlogo': 1,
             'warranty': 100000,
             'wildcard': 0},
            {'category': {'id': 2, 'name': 'pro'},
             'comodo_id': 7,
             'id': 7,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_pro_1_250_0',
             'sgc': 0,
             'trustlogo': 1,
             'warranty': 250000,
             'wildcard': 0},
            {'category': {'id': 2, 'name': 'pro'},
             'comodo_id': 35,
             'id': 8,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_pro_w_250_0',
             'sgc': 0,
             'trustlogo': 1,
             'warranty': 250000,
             'wildcard': 1},
            {'category': {'id': 2, 'name': 'pro'},
             'comodo_id': 323,
             'id': 9,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_pro_w_250_SGC',
             'sgc': 1,
             'trustlogo': 1,
             'warranty': 250000,
             'wildcard': 1},
            {'category': {'id': 3, 'name': 'business'},
             'comodo_id': 337,
             'id': 10,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_bus_1_250_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 250000,
             'wildcard': 0},
            {'category': {'id': 3, 'name': 'business'},
             'comodo_id': 338,
             'id': 11,
             'max_domains': 1,
             'min_domains': 1,
             'name': 'cert_bus_1_250_SGC',
             'sgc': 1,
             'trustlogo': 0,
             'warranty': 250000,
             'wildcard': 0},
            {'category': {'id': 1, 'name': 'standard'},
             'comodo_id': 279,
             'id': 12,
             'max_domains': 5,
             'min_domains': 1,
             'name': 'cert_std_5_0_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 0,
             'wildcard': 0},
            {'category': {'id': 1, 'name': 'standard'},
             'comodo_id': 279,
             'id': 13,
             'max_domains': 10,
             'min_domains': 1,
             'name': 'cert_std_10_0_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 0,
             'wildcard': 0},
            {'category': {'id': 1, 'name': 'standard'},
             'comodo_id': 279,
             'id': 14,
             'max_domains': 20,
             'min_domains': 1,
             'name': 'cert_std_20_0_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 0,
             'wildcard': 0},
            {'category': {'id': 3, 'name': 'business'},
             'comodo_id': 410,
             'id': 15,
             'max_domains': 3,
             'min_domains': 1,
             'name': 'cert_bus_3_250_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 250000,
             'wildcard': 0},
            {'category': {'id': 3, 'name': 'business'},
             'comodo_id': 410,
             'id': 16,
             'max_domains': 5,
             'min_domains': 1,
             'name': 'cert_bus_5_250_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 250000,
             'wildcard': 0},
            {'category': {'id': 3, 'name': 'business'},
             'comodo_id': 410,
             'id': 17,
             'max_domains': 10,
             'min_domains': 1,
             'name': 'cert_bus_10_250_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 250000,
             'wildcard': 0},
            {'category': {'id': 3, 'name': 'business'},
             'comodo_id': 410,
             'id': 18,
             'max_domains': 20,
             'min_domains': 1,
             'name': 'cert_bus_20_250_0',
             'sgc': 0,
             'trustlogo': 0,
             'warranty': 250000,
             'wildcard': 0}
            ]


def list(options):
    ret = [{'trustlogo': False,
            'assumed_name': None,
            'package': 'cert_std_1_0_0',
            'order_number': None,
            'altnames': [],
            'trustlogo_token': {'mydomain.name': 'adadadadad'},
            'date_incorporation': None,
            'card_pay_trustlogo': False,
            'contact': 'TEST1-GANDI',
            'date_start': None,
            'ida_email': None,
            'business_category': None,
            'cert': None,
            'date_end': None,
            'status': 'pending',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_updated': DateTime('20140904T14:06:26'),
            'software': 2,
            'id': 701,
            'joi_locality': None,
            'date_created': DateTime('20140904T14:06:26'),
            'cn': 'mydomain.name',
            'altname': [],
            'sha_version': 1,
            'middleman': '',
            'ida_tel': None,
            'ida_fax': None,
            'comodo_id': None,
            'joi_country': None,
            'joi_state': None},
           {'trustlogo': False,
            'assumed_name': None,
            'package': 'cert_std_1_0_0',
            'order_number': None,
            'altnames': [],
            'trustlogo_token': {'bew.web': 'adadadadad'},
            'date_incorporation': None,
            'card_pay_trustlogo': False,
            'contact': 'TEST1-GANDI',
            'date_start': None,
            'ida_email': None,
            'business_category': None,
            'date_end': None,
            'status': 'pending',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_updated': DateTime('20140904T14:06:26'),
            'software': 2,
            'id': 771,
            'joi_locality': None,
            'date_created': DateTime('20140904T14:06:26'),
            'cn': 'bew.web',
            'altname': [],
            'sha_version': 1,
            'middleman': '',
            'ida_tel': None,
            'ida_fax': None,
            'comodo_id': None,
            'joi_country': None,
            'joi_state': None},
           {'trustlogo': False,
            'assumed_name': None,
            'package': 'cert_pro_1_100_SGC',
            'order_number': None,
            'altnames': [],
            'trustlogo_token': {'iheartcli.com': 'adadadadad'},
            'date_incorporation': None,
            'card_pay_trustlogo': False,
            'contact': 'TEST1-GANDI',
            'date_start': None,
            'ida_email': None,
            'business_category': None,
            'cert': None,
            'date_end': None,
            'status': 'valid',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_updated': DateTime('20140904T14:06:26'),
            'software': 2,
            'id': 709,
            'joi_locality': None,
            'date_created': DateTime('20140904T14:06:26'),
            'cn': 'iheartcli.com',
            'altname': [],
            'sha_version': 1,
            'middleman': '',
            'ida_tel': None,
            'ida_fax': None,
            'comodo_id': None,
            'joi_country': None,
            'joi_state': None},
           {'trustlogo': False,
            'assumed_name': None,
            'package': 'cert_pro_1_100_SGC',
            'order_number': None,
            'altnames': [],
            'trustlogo_token': {'cat.lol': 'adadadadad'},
            'date_incorporation': None,
            'card_pay_trustlogo': False,
            'contact': 'TEST1-GANDI',
            'date_start': None,
            'ida_email': None,
            'business_category': None,
            'cert': None,
            'date_end': None,
            'status': 'valid',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_updated': DateTime('20140904T14:06:26'),
            'software': 2,
            'id': 709,
            'joi_locality': None,
            'date_created': DateTime('20140904T14:06:26'),
            'cn': 'cat.lol',
            'altname': [],
            'sha_version': 1,
            'middleman': '',
            'ida_tel': None,
            'ida_fax': None,
            'comodo_id': None,
            'joi_country': None,
            'joi_state': None},
           {'trustlogo': False,
            'assumed_name': None,
            'package': 'cert_bus_1_250_0',
            'order_number': None,
            'altnames': [],
            'trustlogo_token': {'iheartcli.com': 'adadadadad'},
            'date_incorporation': None,
            'card_pay_trustlogo': False,
            'contact': 'TEST1-GANDI',
            'date_start': None,
            'ida_email': None,
            'business_category': None,
            'cert': None,
            'date_end': None,
            'status': 'valid',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_updated': DateTime('20140904T14:06:26'),
            'software': 2,
            'id': 769,
            'joi_locality': None,
            'date_created': DateTime('20140904T14:06:26'),
            'cn': 'iheartcli.com',
            'altname': [],
            'sha_version': 1,
            'middleman': '',
            'ida_tel': None,
            'ida_fax': None,
            'comodo_id': None,
            'joi_country': None,
            'joi_state': None},
           {'trustlogo': False,
            'assumed_name': None,
            'package': 'cert_bus_20_250_0',
            'order_number': None,
            'altnames': [],
            'trustlogo_token': {'inter.net': 'adadadadad'},
            'date_incorporation': None,
            'card_pay_trustlogo': False,
            'contact': 'TEST1-GANDI',
            'date_start': None,
            'ida_email': None,
            'business_category': None,
            'cert': None,
            'date_end': None,
            'status': 'valid',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_updated': DateTime('20140904T14:06:26'),
            'software': 2,
            'id': 706,
            'joi_locality': None,
            'date_created': DateTime('20140904T14:06:26'),
            'cn': 'inter.net',
            'altname': [],
            'sha_version': 1,
            'middleman': '',
            'ida_tel': None,
            'ida_fax': None,
            'comodo_id': None,
            'joi_country': None,
            'joi_state': None},
           {'altnames': ['pouet.lol.cat'],
            'assumed_name': None,
            'business_category': None,
            'card_pay_trustlogo': False,
            'cert': 'MIIE5zCCA8+gAwIBAgIQAkC4TU9JG8wqhf4FCrsNGTANBgkqhkiG9'
                    'w0BAQsFADBfMQswCQYDVQQGEwJGUjEOMAwGA1UECBMFUGFyaXMxDj'
                    '...'
                    'tU6XzbS6/s2D1/N1wWOOCD/V3XAROtKr1a0mtJ8n7SZyzr0j3weRbN'
                    '7nV24RDQ6d4+GHy5zZstKyDrTknluyyZuDAAYAQJ+nrL5p1gxVNwj1'
                    'f5XKFk=',
            'cn': 'lol.cat',
            'altname': [],
            'comodo_id': 1777348256,
            'contact': 'DF2975-GANDI',
            'csr': '-----BEGIN CERTIFICATE REQUEST-----\n'
                   'MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX'
                   '...'
                   'K+I=\n-----END CERTIFICATE REQUEST-----',
            'date_created': DateTime('20150318T00:00:00'),
            'date_end': DateTime('20160318T00:00:00'),
            'date_incorporation': None,
            'date_start': DateTime('20150318T00:00:00'),
            'date_updated': DateTime('20150318T00:00:00'),
            'id': 710,
            'ida_email': None,
            'ida_fax': None,
            'ida_tel': None,
            'joi_country': None,
            'joi_locality': None,
            'joi_state': None,
            'middleman': '',
            'order_number': 12345678,
            'package': 'cert_std_1_0_0',
            'sha_version': 2,
            'software': 2,
            'status': 'valid',
            'trustlogo': False,
            'trustlogo_token': {'lol.cat': 'ababababa'}}]

    options.pop('items_per_page', None)

    def compare(hc, option):
        if isinstance(option, (type_list, tuple)):
            return hc in option
        return hc == option

    for fkey in options:
        ret = [hc for hc in ret if compare(hc[fkey], options[fkey])]

    return ret


def info(id):
    cert = dict([(cert['id'], cert) for cert in list({})])
    return cert[id]


def create(*args):
    return {'id': 1}


def update(*args):
    return {'id': 400}


def change_dcv(oper_id, dcv_method):
    return True


def resend_dcv(oper_id):
    return True


def get_dcv_params(params):
    return {
        'sha1': 'AD6A9D35FF5BD9FB03A41F4F82CAA4B77',
        'dcv_method': 'dns',
        'fqdns': ['lol.cat'],
        'message': ['920F78CCE11DA7D9554.lol.cat. 10800 IN '
                    'CNAME AD6A9D35FF5BD9FB03A41.comodoca.com.'],
        'dns_records': ['920F78CCE11DA7D9554.lol.cat. 10800 IN '
                        'CNAME AD6A9D35FF5BD9FB03A41.comodoca.com.'],
        'md5': '920F78CCE11DA7ADAD9554',
    }


def delete(cert_id):
    return {'id': 200, 'step': 'WAIT'}


def hosted_list(options):
    fqdns_id = {'test1.domain.fr': [1, 2],
                'test2.domain.fr': [3],
                'test3.domain.fr': [4],
                'test4.domain.fr': [5],
                '*.domain.fr': [6]}

    ret = [{'date_created': DateTime('20150407T00:00:00'),
            'date_expire': DateTime('20160316T00:00:00'),
            'id': 1,
            'state': u'created',
            'subject': u'/OU=Domain Control Validated/OU=Gandi Standard '
                       'SSL/CN=test1.domain.fr'},
           {'date_created': DateTime('20150407T00:00:00'),
            'date_expire': DateTime('20160316T00:00:00'),
            'id': 2,
            'state': u'created',
            'subject': u'/OU=Domain Control Validated/OU=Gandi Standard '
                       'SSL/CN=test1.domain.fr'},
           {'date_created': DateTime('20150408T00:00:00'),
            'date_expire': DateTime('20160408T00:00:00'),
            'id': 3,
            'state': u'created',
            'subject': u'/OU=Domain Control Validated/OU=Gandi Standard '
                       'SSL/CN=test2.domain.fr'},
           {'date_created': DateTime('20150408T00:00:00'),
            'date_expire': DateTime('20160408T00:00:00'),
            'id': 4,
            'state': u'created',
            'subject': u'/OU=Domain Control Validated/OU=Gandi Standard '
                       'SSL/CN=test3.domain.fr'},
           {'date_created': DateTime('20150408T00:00:00'),
            'date_expire': DateTime('20160408T00:00:00'),
            'id': 5,
            'state': u'created',
            'subject': u'/OU=Domain Control Validated/OU=Gandi Standard '
                       'SSL/CN=test4.domain.fr'},
           {'date_created': DateTime('20150409T00:00:00'),
            'date_expire': DateTime('20160409T00:00:00'),
            'id': 6,
            'state': u'created',
            'subject': u'/OU=Domain Control Validated/OU=Gandi Standard '
                       'Wildcard SSL/CN=*.domain.fr'}]

    options.pop('items_per_page', None)
    fqdns = options.pop('fqdns', None)

    if fqdns:
        if not isinstance(fqdns, (type_list, tuple)):
            fqdns = [fqdns]
        for fqdn in fqdns:
            options.setdefault('id', []).extend(fqdns_id.get(fqdn, []))

    def compare(hc, option):
        if isinstance(option, (type_list, tuple)):
            return hc in option
        return hc == option

    for fkey in options:
        ret = [hc for hc in ret if compare(hc[fkey], options[fkey])]

    return ret


def hosted_info(id):
    additionals = {
        1: {'fqdns': [{'type': u'cn', 'name': u'test1.domain.fr'}],
            'related_vhosts': [{'service_id': 1,
                                'type': 'paas',
                                'id': 1,
                                'name': 'test1.domain.fr'}]},
        2: {'fqdns': [{'type': u'cn', 'name': u'test1.domain.fr'}],
            'related_vhosts': [{'service_id': 1,
                                'type': 'paas',
                                'id': 1,
                                'name': 'test1.domain.fr'}]},
        3: {'fqdns': [{'type': u'cn', 'name': u'test2.domain.fr'}],
            'related_vhosts': []},
        4: {'fqdns': [{'type': u'cn', 'name': u'test3.domain.fr'}],
            'related_vhosts': []},
        5: {'fqdns': [{'type': u'cn', 'name': u'test4.domain.fr'}],
            'related_vhosts': []},
        6: {'fqdns': [{'type': u'cn', 'name': u'*.domain.fr'}],
            'related_vhosts': [{'service_id': 2,
                                'type': 'paas',
                                'id': 2,
                                'name': '*.domain.fr'}]}}

    def additional(hc):
        hc.update(additionals[hc['id']])
        return hc

    hc = dict([(hc['id'], additional(hc)) for hc in hosted_list({})])
    return hc.get(id)


def hosted_create(params):
    return hosted_info(5)


def hosted_delete(id):
    return



""" PaaS instances namespace commands. """

import click
from click.exceptions import UsageError

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import (
    output_paas, output_generic, randomstring,
    DatacenterLimited
)
from gandi.cli.core.params import (
    pass_gandi, DATACENTER, SNAPSHOTPROFILE_PAAS, PAAS_TYPE, option,
)


@cli.group(name='paas')
@pass_gandi
def paas(gandi):
    """Commands related to Simple Hosting."""


@paas.command()
@click.option('--state', default=None, help='Filter results by state.')
@click.option('--id', help='Display ids.', is_flag=True)
@click.option('--vhosts', help='Display vhosts.', default=True, is_flag=True)
@click.option('--type', help='Display types.', is_flag=True)
@click.option('--limit', help='Limit number of results.', default=100,
              show_default=True)
@pass_gandi
def list(gandi, state, id, vhosts, type, limit):
    """List PaaS instances."""
    options = {
        'items_per_page': limit,
    }
    if state:
        options['state'] = state

    output_keys = ['name', 'state']
    if id:
        output_keys.append('id')
    if vhosts:
        output_keys.append('vhost')
    if type:
        output_keys.append('type')

    paas_hosts = {}
    result = gandi.paas.list(options)
    for num, paas in enumerate(result):
        paas_hosts[paas['id']] = []
        if vhosts:
            list_vhost = gandi.vhost.list({'paas_id': paas['id']})
            for host in list_vhost:
                paas_hosts[paas['id']].append(host['name'])

        if num:
            gandi.separator_line()
        output_paas(gandi, paas, [], paas_hosts[paas['id']],
                    output_keys)

    return result


@paas.command()
@click.argument('resource')
@click.option('--stat', default=False, is_flag=True,
              help='Display cached page statistic based on the last 24 hours.')
@pass_gandi
def info(gandi, resource, stat):
    """Display information about a PaaS instance.

    Resource can be a vhost, a hostname, or an ID
    Cache statistics are based on 24 hours data.
    """
    output_keys = ['name', 'type', 'size', 'memory', 'console', 'vhost',
                   'dc', 'sftp_server', 'git_server', 'snapshot']

    paas = gandi.paas.info(resource)
    paas_hosts = []
    list_vhost = gandi.vhost.list({'paas_id': paas['id']})

    df = gandi.paas.quota(paas['id'])
    paas.update({'df': df})

    if stat:
        cache = gandi.paas.cache(paas['id'])
        paas.update({'cache': cache})

    for host in list_vhost:
        paas_hosts.append(host['name'])

    output_paas(gandi, paas, [], paas_hosts, output_keys)

    return paas


@paas.command()
@click.option('--origin', default='gandi', help="Set the origin remote's name")
@click.option('--directory', help='Specify the destination directory')
@click.option('--vhost', required=False, default='default')
@click.argument('name', required=True)
@pass_gandi
def clone(gandi, name, vhost, directory, origin):
    """Clone a remote vhost in a local git repository."""
    if vhost != 'default':
        directory = vhost
    else:
        directory = name if not directory else directory

    return gandi.paas.clone(name, vhost, directory, origin)


@paas.command()
@click.option('--vhost', default='default',
              help="Add a remote for a given instance's vhost to the local "
              "git repository")
@click.option('--remote', default='gandi', help="Specify the remote's name")
@click.argument('name', required=True)
@pass_gandi
def attach(gandi, name, vhost, remote):
    """Add remote for an instance's default vhost to the local git repository.
    """
    return gandi.paas.attach(name, vhost, remote)


@cli.command()
@click.option('--remote', default='gandi', help="Specify the remote's name")
@click.option('--branch', default='master',
              help="Specify the branch to deploy")
@pass_gandi
def deploy(gandi, remote, branch):
    """Deploy code on the instance's remote vhost corresponding to current
    directory.
    """
    return gandi.paas.deploy(remote, branch)


@paas.command()
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@click.argument('resource', nargs=-1, required=True)
@pass_gandi
def delete(gandi, background, force, resource):
    """Delete a PaaS instance.

    Resource can be a vhost, a hostname, or an ID
    """
    output_keys = ['id', 'type', 'step']

    possible_resources = gandi.paas.resource_list()
    for item in resource:
        if item not in possible_resources:
            gandi.echo('Sorry PaaS instance %s does not exist' % item)
            gandi.echo('Please use one of the following: %s' %
                       possible_resources)
            return

    if not force:
        instance_info = "'%s'" % ', '.join(resource)
        proceed = click.confirm("Are you sure to delete PaaS instance %s?" %
                                instance_info)

        if not proceed:
            return

    opers = gandi.paas.delete(resource, background)
    if background:
        for oper in opers:
            output_generic(gandi, oper, output_keys)

    return opers


@paas.command()
@click.option('--name', default=None,
              help='Name of the PaaS instance, will be generated if not '
                   'provided.')
@option('--size', default='s',
        type=click.Choice(['s', 's+', 'm', 'l', 'xl', 'xxl']),
        help='Size of the PaaS instance.')
@option('--type', default='pythonpgsql',
        type=PAAS_TYPE,
        help='Type of the PaaS instance.')
@option('--quantity', default=0,
        help='Additional disk amount (in GB).')
@option('--duration', default='1m',
        help='Number of month, suffixed with m.')
@option('--datacenter', type=DATACENTER, default='LU-BI1',
        help='Datacenter where the PaaS will be spawned.')
@click.option('--vhosts', default=None,
              help='Virtual host(s) to be linked to the instance.')
@click.option('--ssl', help='Get ssl on that vhost.', is_flag=True)
@click.option('--private-key', '--pk',
              help='Private key used to generate the ssl Certificate.')
@click.option('--poll-cert', help='Will wait for the certificate creation.',
              is_flag=True)
@click.option('--password', help='Use command-line supplied password.')
@click.option('--snapshotprofile', default=None, type=SNAPSHOTPROFILE_PAAS,
              help='Set a snapshot profile associated to this paas disk.')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@option('--sshkey', multiple=True,
        help='Authorize ssh authentication for the given ssh key.')
@pass_gandi
def create(gandi, name, size, type, quantity, duration, datacenter, vhosts,
           password, snapshotprofile, background, sshkey, ssl, private_key,
           poll_cert):
    """Create a new PaaS instance and initialize associated git repository.

    you can specify a configuration entry named 'sshkey' containing
    path to your sshkey file

    $ gandi config set [-g] sshkey ~/.ssh/id_rsa.pub

    or getting the sshkey "my_key" from your gandi ssh keyring

    $ gandi config set [-g] sshkey my_key

    to know which PaaS instance type to use as type

    $ gandi paas types

    """
    try:
        gandi.datacenter.is_opened(datacenter, 'paas')
    except DatacenterLimited as exc:
        gandi.echo('/!\ Datacenter %s will be closed on %s, '
                   'please consider using another datacenter.' %
                   (datacenter, exc.date))

    if not password:
        password = click.prompt('password', hide_input=True,
                                confirmation_prompt=True)

    if not name:
        name = randomstring('paas')

    if vhosts and not gandi.hostedcert.activate_ssl(vhosts,
                                                    ssl,
                                                    private_key,
                                                    poll_cert):
        return

    result = gandi.paas.create(name, size, type, quantity, duration,
                               datacenter, vhosts, password,
                               snapshotprofile, background, sshkey)
    return result


@paas.command()
@click.option('--name', type=click.STRING, default=None,
              help='Name of the PaaS instance.')
@click.option('--size', default=None,
              type=click.Choice(['s', 's+', 'm', 'x', 'xl', 'xxl']),
              help='Size of the PaaS instance.')
@click.option('--quantity', type=click.INT, default=0,
              help='Additional disk amount (in GB).')
@click.option('--password', default=False, is_flag=True,
              help='Password of the PaaS instance.')
@click.option('--sshkey', multiple=True,
              help='Authorize ssh authentication for the given ssh key.')
@click.option('--upgrade', default=False, is_flag=True,
              help='Upgrade the instance to the last system image if needed.')
@click.option('--console', default=None,
              help='Activate or deactivate the Console.')
@click.option('--snapshotprofile', default=None, type=click.INT,
              help='Set a snapshot profile associated to this paas disk.')
@click.option('--reset-mysql-password', default=None,
              help='Reset mysql password for root.')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--delete-snapshotprofile', default=False, is_flag=True,
              help='Remove a snapshot profile associated to this paas disk.')
@pass_gandi
@click.argument('resource')
def update(gandi, resource, name, size, quantity, password, sshkey,
           upgrade, console, snapshotprofile, reset_mysql_password,
           background, delete_snapshotprofile):
    """Update a PaaS instance.

    Resource can be a Hostname or an ID
    """

    if snapshotprofile and delete_snapshotprofile:
        raise UsageError('You must not set snapshotprofile and '
                         'delete-snapshotprofile.')

    pwd = None
    if password:
        pwd = click.prompt('password', hide_input=True,
                           confirmation_prompt=True)

    if delete_snapshotprofile:
        snapshotprofile = ''

    result = gandi.paas.update(resource, name, size, quantity, pwd,
                               sshkey, upgrade, console, snapshotprofile,
                               reset_mysql_password, background)
    if background:
        gandi.pretty_echo(result)

    return result


@paas.command()
@click.argument('resource', nargs=-1, required=True)
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False)')
@pass_gandi
def restart(gandi, resource, background, force):
    """Restart a PaaS instance.

    Resource can be a vhost, a hostname, or an ID
    """
    output_keys = ['id', 'type', 'step']

    possible_resources = gandi.paas.resource_list()
    for item in resource:
        if item not in possible_resources:
            gandi.echo('Sorry PaaS instance %s does not exist' % item)
            gandi.echo('Please use one of the following: %s' %
                       possible_resources)
            return

    if not force:
        instance_info = "'%s'" % ', '.join(resource)
        proceed = click.confirm("Are you sure to restart PaaS instance %s?" %
                                instance_info)

        if not proceed:
            return

    opers = gandi.paas.restart(resource, background)
    if background:
        for oper in opers:
            output_generic(gandi, oper, output_keys)

    return opers


@paas.command()
@pass_gandi
def types(gandi):
    """List types PaaS instances."""
    options = {}
    types = gandi.paas.type_list(options)
    for type_ in types:
        gandi.echo(type_['name'])

    return types


@paas.command()
@click.argument('resource')
@pass_gandi
def console(gandi, resource):
    """Open a console on a PaaS.

    Resource can be a hostname or an ID
    """
    gandi.echo('/!\ Use ~. ssh escape key to exit.')

    gandi.paas.console(resource)



# -*- coding: utf-8 -*-

from .base import CommandTestCase
from gandi.cli.commands import snapshotprofile


class SnapshotprofileTestCase(CommandTestCase):

    def test_list(self):

        result = self.invoke_with_exceptions(snapshotprofile.list, [])

        self.assertEqual(result.output, """\
id           : 1
name         : minimal
kept_total   : 2
target       : vm
----------
id           : 2
name         : full_week
kept_total   : 7
target       : vm
----------
id           : 3
name         : security
kept_total   : 10
target       : vm
----------
id           : 7
name         : paas_normal
kept_total   : 3
target       : paas
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_filter_paas(self):
        args = ['--only-paas']
        result = self.invoke_with_exceptions(snapshotprofile.list, args)

        self.assertEqual(result.output, """\
id           : 7
name         : paas_normal
kept_total   : 3
target       : paas
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_filter_vm(self):
        args = ['--only-vm']
        result = self.invoke_with_exceptions(snapshotprofile.list, args)

        self.assertEqual(result.output, """\
id           : 1
name         : minimal
kept_total   : 2
target       : vm
----------
id           : 2
name         : full_week
kept_total   : 7
target       : vm
----------
id           : 3
name         : security
kept_total   : 10
target       : vm
""")
        self.assertEqual(result.exit_code, 0)

    def test_info(self):
        args = ['security']
        result = self.invoke_with_exceptions(snapshotprofile.info, args)

        self.assertEqual(result.output, """\
id           : 3
name         : security
kept_total   : 10
target       : vm
quota_factor : 2.0
----------
name         : hourly6
kept_version : 3
----------
name         : daily
kept_version : 6
----------
name         : weekly4
kept_version : 1
""")
        self.assertEqual(result.exit_code, 0)



""" Webaccelerator namespace commands. """

import click

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import (
    output_generic, output_json, output_sub_generic,
    DatacenterLimited
)
from gandi.cli.core.params import (
    pass_gandi, BACKEND, DATACENTER, WEBACC_NAME, WEBACC_VHOST_NAME
)


@cli.group(name='webacc')
@pass_gandi
def webacc(gandi):
    """Commands related to hosting web accelerators."""


@webacc.command()
@click.option('--limit', help="Limit the number of results", default=100,
              show_default=True)
@click.option('--format', type=click.Choice(['json', 'pretty-json']),
              required=False, help="Choose the output format")
@pass_gandi
def list(gandi, limit, format):
    """ List webaccelerators """
    options = {
        'items_per_page': limit,
    }

    result = gandi.webacc.list(options)
    if format:
        output_json(gandi, format, result)
        return result

    output_keys = ['name', 'state', 'ssl']
    for num, webacc in enumerate(result):
        if num:
            gandi.separator_line('-', 4)

        webacc['ssl'] = 'Enabled' if webacc['ssl_enable'] else 'Disable'
        output_generic(gandi, webacc, output_keys, justify=14)

        gandi.echo('Vhosts :')
        for vhost in webacc['vhosts']:
            output_vhosts = ['vhost', 'ssl']
            vhost['vhost'] = vhost['name']
            vhost['ssl'] = 'Disable' if vhost['cert_id'] is None else 'Enabled'
            output_sub_generic(gandi, vhost, output_vhosts,
                               justify=14)
            gandi.echo('')

        gandi.echo('Backends :')
        for server in webacc['servers']:
            try:
                ip = gandi.ip.info(server['ip'])
                iface = gandi.iface.info(ip['iface_id'])
                vm_info = gandi.iaas.info(iface['vm_id'])
                server['name'] = vm_info['hostname']
                output_servers = ['name', 'ip', 'port', 'state']
            except Exception:
                warningmsg = ('\tBackend with ip address %s no longer '
                              'exists.\n\tYou should remove it.'
                              % server['ip'])
                gandi.echo(warningmsg)
                output_servers = ['ip', 'port', 'state']
            output_sub_generic(gandi, server, output_servers,
                               justify=14)
            gandi.echo('')
    return result


@webacc.command()
@click.option('--format', type=click.Choice(['json', 'pretty-json']),
              required=False, help="Choose the output format")
@click.argument('resource', type=WEBACC_NAME)
@pass_gandi
def info(gandi, resource, format):
    """ Display information about a webaccelerator """
    result = gandi.webacc.info(resource)
    if format:
        output_json(gandi, format, result)
        return result

    output_base = {
        'name': result['name'],
        'algorithm': result['lb']['algorithm'],
        'datacenter': result['datacenter']['name'],
        'state': result['state'],
        'ssl': 'Disable' if result['ssl_enable'] is False else 'Enabled'
    }
    output_keys = ['name', 'state', 'datacenter', 'ssl', 'algorithm']
    output_generic(gandi, output_base, output_keys, justify=14)

    gandi.echo('Vhosts :')
    for vhost in result['vhosts']:
        output_vhosts = ['vhost', 'ssl']
        vhost['vhost'] = vhost['name']
        vhost['ssl'] = 'None' if vhost['cert_id'] is None else 'Exists'
        output_sub_generic(gandi, vhost, output_vhosts, justify=14)
        gandi.echo('')

    gandi.echo('Backends :')
    for server in result['servers']:
        try:
            ip = gandi.ip.info(server['ip'])
            iface = gandi.iface.info(ip['iface_id'])
            server['name'] = gandi.iaas.info(iface['vm_id'])['hostname']
            output_servers = ['name', 'ip', 'port', 'state']
        except Exception:
            warningmsg = ('\tBackend with ip address %s no longer exists.'
                          '\n\tYou should remove it.' % server['ip'])
            gandi.echo(warningmsg)
            output_servers = ['ip', 'port', 'state']
        output_sub_generic(gandi, server, output_servers, justify=14)
        gandi.echo('')

    gandi.echo('Probe :')
    output_probe = ['state', 'host', 'interval', 'method', 'response',
                    'threshold', 'timeout', 'url', 'window']
    result['probe']['state'] = ('Disable'
                                if result['probe']['enable'] is False
                                else 'Enabled')
    output_sub_generic(gandi, result['probe'], output_probe, justify=14)
    return result


@webacc.command()
@click.option('--datacenter', '-dc', type=DATACENTER,
              help="Datacenter where the webaccelerator will be created",
              required=True)
@click.option('--backend', '-b', type=BACKEND, multiple=True,
              help="Backend to add in the webaccelerator, use ip:port")
@click.option('--port', '-p', type=click.INT, required=False,
              help="set a default port backend if not specified with backend")
@click.option('--vhost', '-v', help="Vhost to add in the webaccelerator",
              multiple=True)
@click.option('--ssl', help='Get ssl on that vhost.', is_flag=True)
@click.option('--private-key', '--pk',
              help='Private key used to generate the ssl Certificate.')
@click.option('--poll-cert', help='Will wait for the certificate creation.',
              is_flag=True)
@click.option('--algorithm', type=click.Choice(['client-ip', 'round-robin']),
              help="Choose the loadbalancer algorithm", default='client-ip')
@click.option('--ssl-enable', is_flag=True,
              help="Activate SSL support on the webaccelerator")
@click.option('--zone-alter', is_flag=True,
              help="Alter the zone file of the domain for the vhost if domains"
              "are registred at Gandi")
@click.argument('name')
@pass_gandi
def create(gandi, name, datacenter, backend, port, vhost, algorithm,
           ssl_enable, zone_alter, ssl, private_key, poll_cert):
    """ Create a webaccelerator """
    try:
        gandi.datacenter.is_opened(datacenter, 'iaas')
    except DatacenterLimited as exc:
        gandi.echo('/!\ Datacenter %s will be closed on %s, '
                   'please consider using another datacenter.' %
                   (datacenter, exc.date))

    backends = backend
    for backend in backends:
        # Check if a port is set for each backend, else set a default port
        if 'port' not in backend:
            if not port:
                backend['port'] = click.prompt('Please set a port for '
                                               'backends. If you want to set '
                                               'different port for each '
                                               'backend, use `-b ip:port`',
                                               type=int)
            else:
                backend['port'] = port

    if vhost and not gandi.hostedcert.activate_ssl(vhost,
                                                   ssl,
                                                   private_key,
                                                   poll_cert):
        return

    result = gandi.webacc.create(name, datacenter, backends, vhost, algorithm,
                                 ssl_enable, zone_alter)
    return result


@webacc.command()
@click.option('--name', '-n', help="The name of the webaccelerator")
@click.option('--algorithm', type=click.Choice(['client-ip', 'round-robin']),
              help="Choose the loadbalancer algorithm")
@click.option('--ssl-enable', is_flag=True,
              help="Activate SSL support on the webaccelerator")
@click.option('--ssl-disable', is_flag=True,
              help="Deactivate SSL support on the webaccelerator")
@click.argument('resource', required=True, type=WEBACC_NAME)
@pass_gandi
def update(gandi, resource, name, algorithm, ssl_enable, ssl_disable):
    """Update a webaccelerator"""
    result = gandi.webacc.update(resource, name, algorithm, ssl_enable,
                                 ssl_disable)
    return result


@webacc.command()
@click.option('--vhost', '-v', help="Remove vhosts in the webaccelerator",
              multiple=True, type=WEBACC_VHOST_NAME)
@click.option('--backend', '-b', help="Remove backends in the webaccelerator",
              multiple=True, type=BACKEND)
@click.option('--port', '-p', type=click.INT, required=False,
              help="The backend port if not specified with backend")
@click.option('--webacc', '-w', type=WEBACC_NAME, required=False,
              help='The webaccelerator name')
@pass_gandi
def delete(gandi, webacc, vhost, backend, port):
    """ Delete a webaccelerator, a vhost or a backend """
    result = []
    if webacc:
        result = gandi.webacc.delete(webacc)

    if backend:
        backends = backend
        for backend in backends:
            if 'port' not in backend:
                if not port:
                    backend['port'] = click.prompt('Please set a port for '
                                                   'backends. If you want to '
                                                   ' different port for '
                                                   'each backend, use `-b '
                                                   'ip:port`', type=int)
                else:
                    backend['port'] = port
            result = gandi.webacc.backend_remove(backend)

    if vhost:
        vhosts = vhost
        for vhost in vhosts:
            result = gandi.webacc.vhost_remove(vhost)

    return result


@webacc.command()
@click.option('--vhost', '-v', help="Add vhosts in the webaccelerator",
              multiple=True)
@click.option('--ssl', help='Get ssl on that vhost.', is_flag=True)
@click.option('--private-key', '--pk',
              help='Private key used to generate the ssl Certificate.')
@click.option('--poll-cert', help='Will wait for the certificate creation.',
              is_flag=True)
@click.option('--zone-alter', is_flag=True,
              help="Alter and activate zone file if Gandi DNS are used for"
                   " the domain",)
@click.option('--backend', '-b', help="Add backends in the webaccelerator",
              type=BACKEND, multiple=True)
@click.option('--port', '-p', type=click.INT, required=False,
              help="set a default port backend if not specified with backend")
@click.argument('resource', type=WEBACC_NAME)
@pass_gandi
def add(gandi, resource, vhost, zone_alter, backend, port, ssl, private_key,
        poll_cert):
    """ Add a backend or a vhost on a webaccelerator """
    result = []
    if backend:
        backends = backend
        for backend in backends:
            # Check if a port is set for each backend, else set a default port
            if 'port' not in backend:
                if not port:
                    backend['port'] = click.prompt('Please set a port for '
                                                   'backends. If you want to '
                                                   ' different port for '
                                                   'each backend, use `-b '
                                                   'ip:port`', type=int)
                else:
                    backend['port'] = port
            result = gandi.webacc.backend_add(resource, backend)

    if vhost:
        if not gandi.hostedcert.activate_ssl(vhost, ssl, private_key,
                                             poll_cert):
            return
        vhosts = vhost
        for vhost in vhosts:
            params = {'vhost': vhost}
            if zone_alter:
                params['zone_alter'] = zone_alter
            result = gandi.webacc.vhost_add(resource, params)
    return result


@webacc.command()
@click.option('--backend', '-b', help="Enable backends in the webaccelerator",
              multiple=True, type=BACKEND)
@click.option('--port', '-p', type=click.INT, required=False,
              help="set a default port backend if not specified with backend")
@click.option('--probe', '-p', help="Enable probe for the webaccelerator",
              is_flag=True)
@click.argument('resource', required=False, type=WEBACC_NAME)
@pass_gandi
def enable(gandi, resource, backend, port, probe):
    """ Enable a backend or a probe on a webaccelerator """
    result = []
    if backend:
        backends = backend
        for backend in backends:
            if 'port' not in backend:
                if not port:
                    backend['port'] = click.prompt('Please set a port for '
                                                   'backends. If you want to '
                                                   ' different port for '
                                                   'each backend, use `-b '
                                                   'ip:port`', type=int)
                else:
                    backend['port'] = port
            result = gandi.webacc.backend_enable(backend)

    if probe:
        if not resource:
            gandi.echo('You need to indicate the Webaccelerator name')
            return
        result = gandi.webacc.probe_enable(resource)

    return result


@webacc.command()
@click.option('--backend', '-b', help="Disable backends in the webaccelerator",
              multiple=True, type=BACKEND)
@click.option('--port', '-p', type=click.INT, required=False,
              help="set a default port backend if not specified with backend")
@click.option('--probe', '-p', help="Disable probe for the webaccelerator",
              is_flag=True)
@click.argument('resource', required=False, type=WEBACC_NAME)
@pass_gandi
def disable(gandi, resource, backend, port, probe):
    """ Disable a backend or a probe on a webaccelerator """
    result = []
    if backend:
        backends = backend
        for backend in backends:
            if 'port' not in backend:
                if not port:
                    backend['port'] = click.prompt('Please set a port for '
                                                   'backends. If you want to '
                                                   ' different port for '
                                                   'each backend, use `-b '
                                                   'ip:port`', type=int)
                else:
                    backend['port'] = port
            result = gandi.webacc.backend_disable(backend)

    if probe:
        if not resource:
            gandi.echo('You need to indicate the Webaccelerator name')
            return
        result = gandi.webacc.probe_disable(resource)

    return result


@webacc.command()
@click.option('--enable', '-e', is_flag=True,
              help="Enable the probe on the webaccelerator")
@click.option('--disable', '-d', is_flag=True,
              help="Disable the probe on the webaccelerator")
@click.option('--test', is_flag=True, help="Test probe on the webaccelerator")
@click.option('--host', '-h', help="Set the host to test")
@click.option('--interval', '-i', help="Set interval for the probe",
              type=click.INT)
@click.option('--http-method', '-m', help="Choose HTTP method for the probe",
              type=click.Choice(['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS']))
@click.option('--http-response', '-r', type=click.INT,
              help="HTTP response code expected by the probe")
@click.option('--threshold', '-t', type=click.INT,
              help="Number of probes to consider in the window")
@click.option('--timeout', help="Timeout in seconds",
              type=click.INT)
@click.option('--url', '-u', help="Probe url in the virtual host",
              type=click.STRING)
@click.option('--window', '-w', type=click.INT,
              help="Total number of probes to consider health decision")
@click.argument('resource', type=WEBACC_NAME)
@pass_gandi
def probe(gandi, resource, enable, disable, test, host, interval, http_method,
          http_response, threshold, timeout, url, window):
    """ Manage a probe for a webaccelerator """
    result = gandi.webacc.probe(resource, enable, disable, test, host,
                                interval, http_method, http_response,
                                threshold, timeout, url, window)
    output_keys = ['status', 'timeout']
    output_generic(gandi, result, output_keys, justify=14)
    return result
